package com.bsn.mes.md.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.bsn.common.annotation.Log;
import com.bsn.common.core.controller.BaseController;
import com.bsn.common.core.domain.AjaxResult;
import com.bsn.common.enums.BusinessType;
import com.bsn.mes.md.domain.MdSaleOrderOffice;
import com.bsn.mes.md.service.IMdSaleOrderOfficeService;
import com.bsn.common.utils.poi.ExcelUtil;
import com.bsn.common.core.page.TableDataInfo;

/**
 * 销售订单办事处Controller
 *
 * @author yinjinlu
 * @date 2024-01-22
 */
@RestController
@RequestMapping("/md/office")
public class MdSaleOrderOfficeController extends BaseController
{
    @Autowired
    private IMdSaleOrderOfficeService mdSaleOrderOfficeService;

    /**
     * 查询销售订单办事处列表
     */
    @PreAuthorize("@ss.hasPermi('md:office:list')")
    @GetMapping("/list")
    public TableDataInfo list(MdSaleOrderOffice mdSaleOrderOffice)
    {
        startPage();
        List<MdSaleOrderOffice> list = mdSaleOrderOfficeService.selectMdSaleOrderOfficeList(mdSaleOrderOffice);
        return getDataTable(list);
    }

    /**
     * 导出销售订单办事处列表
     */
    @PreAuthorize("@ss.hasPermi('md:office:export')")
    @Log(title = "销售订单办事处", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MdSaleOrderOffice mdSaleOrderOffice)
    {
        List<MdSaleOrderOffice> list = mdSaleOrderOfficeService.selectMdSaleOrderOfficeList(mdSaleOrderOffice);
        ExcelUtil<MdSaleOrderOffice> util = new ExcelUtil<MdSaleOrderOffice>(MdSaleOrderOffice.class);
        util.exportExcel(response, list, "销售订单办事处数据");
    }

    /**
     * 获取销售订单办事处详细信息
     */
    @PreAuthorize("@ss.hasPermi('md:office:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(mdSaleOrderOfficeService.selectMdSaleOrderOfficeById(id));
    }

    /**
     * 新增销售订单办事处
     */
    @PreAuthorize("@ss.hasPermi('md:office:add')")
    @Log(title = "销售订单办事处", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody MdSaleOrderOffice mdSaleOrderOffice)
    {
        return toAjax(mdSaleOrderOfficeService.insertMdSaleOrderOffice(mdSaleOrderOffice));
    }

    /**
     * 修改销售订单办事处
     */
    @PreAuthorize("@ss.hasPermi('md:office:edit')")
    @Log(title = "销售订单办事处", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody MdSaleOrderOffice mdSaleOrderOffice)
    {
        return toAjax(mdSaleOrderOfficeService.updateMdSaleOrderOffice(mdSaleOrderOffice));
    }

    /**
     * 删除销售订单办事处
     */
    @PreAuthorize("@ss.hasPermi('md:office:remove')")
    @Log(title = "销售订单办事处", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(mdSaleOrderOfficeService.deleteMdSaleOrderOfficeByIds(ids));
    }
}
