package com.bsn.mes.pro.controller.mobile;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.annotation.Log;
import com.bsn.common.core.controller.BaseController;
import com.bsn.common.core.domain.AjaxResult;
import com.bsn.common.core.page.TableDataInfo;
import com.bsn.common.enums.BusinessType;
import com.bsn.common.utils.data.ExceptionUtil;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.mes.constant.TaskStatusEnum;
import com.bsn.mes.pro.domain.ProFeedback;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.task.*;
import com.bsn.mes.pro.domain.vo.ProPackagePrintResult;
import com.bsn.mes.pro.domain.vo.ProTaskQuery;
import com.bsn.mes.pro.domain.vo.ProTaskVo;
import com.bsn.mes.pro.domain.vo.ProWorkorderQuery;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkorderPackagePrint;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageAddParams;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageInfo;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageQueryParams;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkorderMapper;
import com.bsn.mes.pro.mapper.task.ProTaskAssistProcessMapper;
import com.bsn.mes.pro.mapper.task.ProTaskMapper;
import com.bsn.mes.pro.mapper.task.ProTaskWorkorderMapper;
import com.bsn.mes.pro.mapper.task.ProTaskWorkunitMapper;
import com.bsn.mes.pro.service.IProFeedbackService;
import com.bsn.mes.pro.service.IProMaterialRequestService;
import com.bsn.mes.pro.service.IProRecoilMaterialService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderProcessToolService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkorderService;
import com.bsn.mes.pro.service.task.IProTaskAssistProcessService;
import com.bsn.mes.pro.service.task.IProTaskIssueService;
import com.bsn.mes.pro.service.task.IProTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.collectingAndThen;
import static java.util.stream.Collectors.toCollection;

@Api
@RestController
@RequestMapping("/mobile/pro/protask")
public class ProTaskMobController extends BaseController {

    @Autowired
    private IProTaskService proTaskService;

    @Autowired
    private IProTaskAssistProcessService proTaskAssistProcessService;

    @Autowired
    private IProTaskIssueService proTaskIssueService;
    @Autowired
    private IProMaterialRequestService proMaterialRequestService;
    @Autowired
    private IProRecoilMaterialService proRecoilMaterialService;
    @Autowired
    private IProWorkorderService proWorkorderService;
    @Resource
    private IProWorkOrderProcessToolService proWorkOrderProcessToolService;
    @Resource
    private ProTaskWorkunitMapper proTaskWorkunitMapper;
    @Resource
    ProTaskAssistProcessMapper proTaskAssistProcessMapper;
    @Autowired
    private ProWorkorderMapper proWorkorderMapper;

    @Autowired
    private ProTaskWorkorderMapper proTaskWorkorderMapper;

    @Autowired
    private ProTaskMapper proTaskMapper;
    @Autowired
    private IProFeedbackService proFeedbackService;


    @GetMapping("/getlist")
    public AjaxResult getIssueList(ProTaskIssue proTaskIssue) {
        List<ProTaskIssue> list = proTaskIssueService.selectProTaskIssueList(proTaskIssue);
        return AjaxResult.success(list);
    }

    @ApiOperation("查询状态未完成的生产任务接口")
    @GetMapping("/buttonList/{taskWorkunitIds}")
    public List<Map<String, String>> buttonList(@PathVariable Long[] taskWorkunitIds) {
        ExceptionUtil.checkTrueThrowException(taskWorkunitIds.length == 0, "参数非法");
        List<Map<String, String>> result = new ArrayList<>();
        for (Long workunitId : taskWorkunitIds) {
            boolean b1 = proMaterialRequestService.checkMaterialRequestIsExist(workunitId);
            boolean b2 = proRecoilMaterialService.checkRecoilMaterialIsExist(workunitId);
            boolean b3 = proWorkOrderProcessToolService.checkProcessToolIsExsit(workunitId);

            Map<String, String> map = new HashMap<>();
            map.put("materialView", b1 ? "1" : "0");
            map.put("recoilMaterialView", b2 ? "1" : "0");
            map.put("processToolView", b3 ? "1" : "0");
            map.put("taskWorkunitId", String.valueOf(workunitId));
            result.add(map);
        }
        return result;
    }


    /**
     * 查询工作站的生产任务
     */
    @ApiOperation("查询状态未完成的生产任务接口")
    @GetMapping("/getTaskList")
    public TableDataInfo<ProTask> list(ProTaskQuery proTask) {
        ProTaskQuery proTaskQuery = new ProTaskQuery();
        BeanUtils.copyProperties(proTask, proTaskQuery);
        //判断前端传来是编号还是工单
        if (StringUtils.isNotEmpty(proTask.getArrangeCode())) {
            if (proTask.getArrangeCode().length() == 12) {
                proTask.setWorkorderCode(proTask.getArrangeCode());
                proTask.setArrangeCode(null);
            }
        }

        List<ProTask> rst = new ArrayList<>();
        List<ProTask> list = new ArrayList<>();
        QueryWrapper<ProTask> query = new QueryWrapper<>();
        List<String> statusList = Arrays.asList(TaskStatusEnum.BEGINNING.getStatus(), TaskStatusEnum.PREPARE.getStatus(),
                TaskStatusEnum.PAUSE.getStatus(), TaskStatusEnum.ERROR_STOP.getStatus());
        query.in("ptw.status", statusList);
        query.eq(proTask.getWorkunitId() != null, "mw.workunit_id", proTask.getWorkunitId());
        query.eq(StringUtils.isNotEmpty(proTask.getArrangeCode()), "t.arrange_code", proTask.getArrangeCode());
        query.gt(proTask.getAllowProduce() != null && proTask.getAllowProduce() == 1, "t.quantity_wait", 0);
        query.orderByAsc("statusIndex").orderByAsc("ptw.schedule_start_date");
        //搜索条件为工单号查询相应任务ID作为查询条件
        if (StringUtils.isNotEmpty(proTask.getWorkorderCode())) {
            QueryWrapper<ProTask> taskQuery = new QueryWrapper<>();
            taskQuery.eq("t3.workorder_code", proTask.getWorkorderCode());
            taskQuery.eq(proTask.getWorkunitId() != null, "t4.workunit_id", proTask.getWorkunitId());
            List<ProTaskVo> taskVoList = proTaskMapper.selectProTaskJoinWorkorder(taskQuery);
            if (CollectionUtil.isEmpty(taskVoList)) {
                if (CollectionUtil.isNotEmpty(rst)) {
                    for (ProTask task : rst) {
                        Double v = proFeedbackService.selectQuantityQualifySum(new ProFeedback(task.getArrangeCode(), null, null, task.getTaskWorkunitId()));
                        task.setArrangeFeedbackSum(v);
                    }
                }
                return getDataTable(rst);
            } else {
                List<Long> taskIds = taskVoList.stream().map(ProTaskVo::getTaskId).collect(Collectors.toList());
                query.in("t.task_id", taskIds);
            }
        }

        //判断是否前端有没传来工单或者是编码
        if (StringUtils.isNotEmpty(proTaskQuery.getArrangeCode())) {
            QueryWrapper<ProTask> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("ptw.status", statusList);
            queryWrapper.eq(proTask.getWorkunitId() != null, "mw.workunit_id", proTask.getWorkunitId());
            queryWrapper.gt(proTask.getAllowProduce() != null && proTask.getAllowProduce() == 1, "t.quantity_wait", 0);
            queryWrapper.orderByAsc("statusIndex").orderByAsc("ptw.schedule_start_date");

            List<ProTask> proTasks = proTaskService.selectProTaskJoinTaskWorkUnit(queryWrapper);
            List<ProTask> proTasks1 = proTaskService.selectProTaskJoinTaskWorkUnit(query);
            for (ProTask task : proTasks1) {
                for (ProTask task1 : proTasks) {
                    if (task.getTaskId().equals(task1.getTaskId())) {
                        task1.setLocation(1L);
                    }
                }
            }
            list.addAll(proTasks);
        } else {
            list = proTaskService.selectProTaskJoinTaskWorkUnit(query);
        }

        rst = list.stream().filter(t -> !TaskStatusEnum.FINISHED.getStatus().equals(t.getStatus())).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(rst)) {
            for (ProTask task : rst) {
                String arrangeCode = task.getArrangeCode();
                if (StringUtils.isNotEmpty(arrangeCode)) {
                    ProWorkorderQuery workorderQuery = new ProWorkorderQuery();
                    workorderQuery.setArrangeCode(arrangeCode);
                    List<ProWorkorder> proWorkorders = proWorkorderService.selectProWorkorderList(workorderQuery);
                    task.setProWorkorderList(proWorkorders);

                    Double v = proFeedbackService.selectQuantityQualifySum(new ProFeedback(arrangeCode, null, null, task.getTaskWorkunitId()));
                    task.setArrangeFeedbackSum(v);
                }
            }
        }

        Set<Long> taskWorkunitIds = rst.stream().map(ProTask::getTaskWorkunitId).collect(Collectors.toSet());
        if (CollectionUtil.isNotEmpty(taskWorkunitIds)) {
            Map<Long, ProTaskAssistProcess> assistProcessMap = proTaskAssistProcessMapper.selectListByQw(new QueryWrapper<ProTaskAssistProcess>().in("task_workunit_id", taskWorkunitIds)).stream().collect(Collectors.toMap(ProTaskAssistProcess::getTaskWorkunitId, x -> x));

            List<ProTask> proTasks = proTaskWorkunitMapper.selectListIsExistToolByTaskWorkunitIds(new QueryWrapper<ProTaskWorkunit>().in("task_workunit_id", taskWorkunitIds));
            Map<Long, ProTask> isExistToolList = proTasks.stream().collect(Collectors.toMap(ProTask::getTaskWorkunitId, x -> x));

            for (ProTask task : rst) {
                ProTaskAssistProcess assistProcess = assistProcessMap.get(task.getTaskWorkunitId());
                ProTask taskVal = isExistToolList.get(task.getTaskWorkunitId());

                if (taskVal != null) {
                    task.setIsExistTool(taskVal.getIsExistTool());
                }

                if (assistProcess != null) {
                    task.setCloseType(assistProcess.getCloseType());
                    task.setWorkorderCode(assistProcess.getWorkorderCode());
                }
            }


        }


        return getDataTable(rst);
    }

    /**
     * 根据TaskCode获取生产任务详情
     *
     * @return
     */
    @GetMapping("/getProTaskByTaskCode")
    public AjaxResult getProTaskByTaskCode(@RequestParam("taskCode") String taskCode) {
        return AjaxResult.success(proTaskService.selectProTaskByTaskCode(taskCode));
    }

    /**
     * 获取生产任务详细信息
     */
    @ApiOperation("查询生产任务详情接口")
    @PreAuthorize("@ss.hasPermi('mes:pro:protask:query')")
    @GetMapping(value = "/{taskId}")
    public AjaxResult getInfo(@PathVariable("taskId") Long taskId) {
        return AjaxResult.success(proTaskService.selectProTaskByTaskId(taskId));
    }


    /**
     * 修改生产任务状态
     */
    @ApiOperation("修改生产任务状态接口")
    @Log(title = "生产任务", businessType = BusinessType.UPDATE)
    @PostMapping("/change")
    @ResponseBody
    public AjaxResult changeStatus(ProTask proTask) {
        proTaskService.changeStatus(proTask);
        return AjaxResult.success();
    }


    @ApiOperation("生产报工")
    @Log(title = "生产报工", businessType = BusinessType.INSERT)
    @PostMapping("/feedback")
    @ResponseBody
    public AjaxResult<List<ProPackagePrintResult>> feedBack(ProFeedback feedback) {
        return AjaxResult.success(proTaskService.feedBack(feedback));
    }

    /**
     * 修改工序任务外协关系
     */
    @Log(title = "外协报工", businessType = BusinessType.INSERT)
    @RequestMapping(value = "/assistFeedback", method = RequestMethod.POST)
    public AjaxResult assistFeedback(@RequestBody ProFeedback proFeedback) {
        return toAjax(proTaskAssistProcessService.feedBack(proFeedback));
    }

    @ApiOperation("包装工序生产报工")
    @Log(title = "包装工序生产报工", businessType = BusinessType.INSERT)
    @PostMapping("/feedBackPackage")
    @ResponseBody
    public AjaxResult<List<WorkorderPackagePrint>> feedBackPackage(FeedbackPackageAddParams feedback) {
        return AjaxResult.success(proTaskService.feedBackPackage(feedback));
    }

    @ApiOperation("包装工序报工列表")
    @PostMapping("/listFeedBackPackage")
    @ResponseBody
    public AjaxResult<FeedbackPackageInfo> listFeedBackPackage(@RequestBody FeedbackPackageQueryParams query) {
        if (query.getMergePack() == 1 && (query.getPackNum() == null || query.getPackNum() == 0)) {
            query.setPackNum(Integer.MAX_VALUE);
        }
        FeedbackPackageInfo rst = proTaskService.listFeedBackPackage(query);
        return AjaxResult.success(rst);
    }

    /**
     * pad端查看图片按钮显示的工单
     *
     * @return
     */
    @GetMapping("/getProWorkder")
    public AjaxResult getProWorkderTaskID(@RequestParam("taskId") Long taskId) {
        List<ProWorkorder> workorderList = new ArrayList<>();
        ProTaskWorkorder proTaskWorkorder = new ProTaskWorkorder();
        proTaskWorkorder.setTaskId(taskId);
        List<ProTaskWorkorder> proTaskWorkorders = proTaskWorkorderMapper.selectProTaskWorkorderList(proTaskWorkorder);
        for (ProTaskWorkorder pro : proTaskWorkorders) {
            QueryWrapper<ProWorkorder> proWorkorderQueryWrapper = new QueryWrapper<>();
            proWorkorderQueryWrapper.eq("workorder_id", pro.getWorkorderId());
            ProWorkorder proWorkorder = proWorkorderMapper.selectListByQw(proWorkorderQueryWrapper).get(0);
            workorderList.add(proWorkorder);
        }
        List<ProWorkorder> collect = workorderList.stream().collect(
                collectingAndThen(
                        toCollection(() -> new TreeSet<>(Comparator.comparing(ProWorkorder::getProductName))),
                        ArrayList::new));

        return AjaxResult.success(collect);
    }

    /**
     * 查看下一道工序
     */
    @GetMapping("/feedBackNextProcess")
    public AjaxResult feedbackNextProcess(ProTask protask) {
        ProTask task = proTaskService.selectProTaskByTaskId(protask.getTaskId());
        ProTask task1 = new ProTask();
        QueryWrapper<ProTask> query = new QueryWrapper<>();
        query.notIn("task_id", protask.getTaskId());
        query.eq("task_batch", task.getTaskBatch());
        query.ge("ordinal", task.getOrdinal());
        query.orderByAsc("ordinal");
        List<ProTask> proTasks = proTaskMapper.selectProTaskJoinProcess(query);
        if (CollUtil.isNotEmpty(proTasks)) {
            task1 = proTasks.get(0);
            if (Objects.nonNull(task1)) {
                ProTask proTask = proTaskMapper.selectWorkunitTaskId(task1.getTaskId());
                task1.setWorkunitName(proTask.getWorkunitName());
            }
        }
        return AjaxResult.success(task1);
    }


}
