package com.bsn.mes.pro.schedule.impl;

import cn.hutool.core.date.StopWatch;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.exception.ServiceException;
import com.bsn.mes.constant.TaskStatusEnum;
import com.bsn.mes.constant.WorkorderStatusEnum;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.task.ProTask;
import com.bsn.mes.pro.domain.task.ProTaskWorkorder;
import com.bsn.mes.pro.domain.task.ProTaskWorkunit;
import com.bsn.mes.pro.schedule.AlgorithmResultProcess;
import com.bsn.mes.pro.schedule.Job;
import com.bsn.mes.pro.schedule.ScheduleStatus;
import com.bsn.mes.pro.schedule.Task;
import com.bsn.mes.pro.service.IProMaterialRequestService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkorderService;
import com.bsn.mes.pro.service.task.IProTaskService;
import com.bsn.mes.pro.service.task.IProTaskWorkorderService;
import com.bsn.mes.pro.service.task.IProTaskWorkunitService;
import com.bsn.mes.tm.service.ITmToolRequestService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

@Component
public class AlgorithmResultProcessImpl implements AlgorithmResultProcess {
 
    @Autowired
    IProTaskService proTaskService;
    @Autowired
    IProTaskWorkunitService proTaskWorkunitService;
    @Autowired
    IProTaskWorkorderService proTaskWorkorderService;
    @Autowired
    IProWorkorderService proWorkorderService;
    @Autowired
    private ITmToolRequestService tmToolRequestService;

    @Autowired
    private IProMaterialRequestService proMaterialRequestService;

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Transactional
    @Override
    public void execute(LocalDateTime scheduleStartDate, List<Job> jobs) {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        for (Job job : jobs) {
            if (!ScheduleStatus.Success.equals(job.getScheduleStatus()) &&
                    !ScheduleStatus.Delay.equals(job.getScheduleStatus())) {
                throw new ServiceException(String.format("排产失败%s，%s，%s", job.getScheduleStatus()
                        .getDescription(), job.getId(), job.getScheduleDesc()));
            }
            int i = 1;
            for (Task task : job.getTasks()) {
                //保存任务表
                ProTask proTask = this.convertTask(job, task);
                proTask.setArrangeCode(job.getId());
                proTask.setTaskBatch(proTask.getArrangeCode()+"-001");
                proTask.setOrdinal(i);
                //首工序，待加工数等于排产数
                if (i == 1) {
                    proTask.setQuantityWait(task.getOrderQuantity());
                } else {
                    proTask.setQuantityWait(BigDecimal.ZERO);
                }
                //末道工序，打上标记
                if (i == job.getTasks().size()) {
                    proTask.setIsLastProcess(1);
                } else {
                    proTask.setIsLastProcess(0);
                }
                proTaskService.insertProTask(proTask);
                long j = 1;
                for (ProWorkorder workorder : job.getJobExtend().getWorkorderList()) {
                    ProTaskWorkorder proTaskWorkorder = new ProTaskWorkorder();
                    proTaskWorkorder.setTaskId(proTask.getTaskId());
                    proTaskWorkorder.setIdx(j);
                    proTaskWorkorder.setWorkorderId(workorder.getWorkorderId());
                    // 保存
                    proTaskWorkorderService.insertProTaskWorkorder(proTaskWorkorder);
                    //更新工单状态
                    ProWorkorder proWorkorderUpdate = new ProWorkorder();
                    proWorkorderUpdate.setStatus(WorkorderStatusEnum.ISSUED.getValue());
                    proWorkorderUpdate.setArrangeCode(workorder.getArrangeCode());
                    proWorkorderUpdate.setWorkorderId(workorder.getWorkorderId());
                    proWorkorderService.updateProWorkorder(proWorkorderUpdate);
                    j++;
                }
                List<Long> taskWorkunitIds = new ArrayList<>();
                //保存派工数据
                List<Task> dispenseList = task.getSplitTask();
                if (dispenseList == null) {
                    ProTaskWorkunit taskWorkunit = this.convertTaskWorkunit(scheduleStartDate, proTask, task);
                    proTaskWorkunitService.insertProTaskWorkunit(taskWorkunit);
                    taskWorkunitIds.add(taskWorkunit.getTaskWorkunitId());
                } else {
                    for (Task dispense : dispenseList) {
                        ProTaskWorkunit taskWorkunit = this.convertTaskWorkunit(scheduleStartDate, proTask, dispense);
                        proTaskWorkunitService.insertProTaskWorkunit(taskWorkunit);
                        taskWorkunitIds.add(taskWorkunit.getWorkunitId());
                    }
                }

                i++;
                //生产领料申请单
                taskWorkunitIds.forEach(id->{
                    proMaterialRequestService.insertProMaterialRequestIssueItem(id, proTask);
                });

                // 生成刀模版具
                QueryWrapper<ProTaskWorkunit> queryWrapper = new QueryWrapper<>();
                queryWrapper.in("task_workunit_id", taskWorkunitIds);
                List<ProTaskWorkunit> proTaskWorkunits = proTaskWorkunitService.selectListByQw(queryWrapper);
                tmToolRequestService.addedToolRequestAppForm(proTaskWorkunits);
            }


        }
        stopWatch.stop();
        logger.info(String.format("排产结果保存、刀具/物料申请单生成耗时：%s", stopWatch.getTotalTimeSeconds()));

    }

    private ProTask convertTask(Job job, Task task) {
        ProTask proTask = new ProTask();
        proTask.setProcessId(task.getProcessId());
        proTask.setProcessName(task.getProcessName());
        proTask.setQuantity(task.getOrderQuantity());
        proTask.setWorkstationId(task.getScheduledEquipment().getWorkCenterId());
        proTask.setWorkstationCode(task.getScheduledEquipment().getWorkCenterCode());
        proTask.setWorkstationName(task.getScheduledEquipment().getWorkCenterName());
        return proTask;
    }

    private ProTaskWorkunit convertTaskWorkunit(LocalDateTime scheduleStartDate, ProTask proTask, Task task) {
        ProTaskWorkunit taskWorkunit = new ProTaskWorkunit();
        taskWorkunit.setTaskId(proTask.getTaskId());
        taskWorkunit.setIdx(1L);
        taskWorkunit.setWorkunitId(task.getScheduledEquipment().getId());
        taskWorkunit.setWorkunitCode(task.getScheduledEquipment().getWorkUnitCode());
        taskWorkunit.setWorkunitName(task.getScheduledEquipment().getWorkUnitName());
        taskWorkunit.setQuantity(task.getOrderQuantity());
        taskWorkunit.setQuantityProduced(BigDecimal.ZERO);
        taskWorkunit.setQuantityQualify(BigDecimal.ZERO);
        taskWorkunit.setQuantityUnqualify(BigDecimal.ZERO);
        taskWorkunit.setStatus(TaskStatusEnum.PREPARE.getStatus());
        LocalDateTime sdate = scheduleStartDate.plus(task.getScheduledStartedTime());
        LocalDateTime edate = scheduleStartDate.plus(task.getScheduledEndedTime());
        Date sdate1 = java.util.Date.from(sdate.atZone(ZoneId.systemDefault()).toInstant());
        Date edate1 = java.util.Date.from(edate.atZone(ZoneId.systemDefault()).toInstant());
        taskWorkunit.setScheduleStartDate(sdate1);
        taskWorkunit.setScheduleEndDate(edate1);
        return taskWorkunit;
    }

}
