package com.bsn.mes.pro.schedule.strategy;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.NumberUtil;
import com.bsn.mes.pro.schedule.Equipment;
import com.bsn.mes.pro.schedule.IOperationTimeCalculator;
import com.bsn.mes.pro.schedule.TaskSchedulingContext;

import java.time.Duration;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 排产时间评估设备
 * 交期内生产=5、交期外生产=0
 * 时间先后=5
 * NumberUtil.max(6-排名*1,0)
 *
 */
public class EvaluateEquipmentDeliveryTime implements EvaluateEquipment{
    @Override
    public double evaluate(TaskSchedulingContext context, IOperationTimeCalculator.OperationTimePlan operationTimePlan
            , List<TaskSchedulingContext.TaskSchedulingResultContext> scheduleResult) {
        double rst = 0;
        //最少换型
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItemList = operationTimePlan.getPlan();
        if(operationTimePlanItemList.isEmpty()){
            return rst;
        }
        Duration lastEndTime = operationTimePlanItemList.get(operationTimePlanItemList.size()-1).getEndedTime();
        //时间评分
        int rank = this.getRank(context.getEquipment(), scheduleResult);
        rst += NumberUtil.max(6-rank*1.0,0);
        // 在交期范围内，5
        // 在交期范围外，0
        if(context.getJob().getProducedDeliveryTime().compareTo(lastEndTime)>=0){
            rst += 5;
        }
        return rst;
    }


    private int getRank(Equipment equipment, List<TaskSchedulingContext.TaskSchedulingResultContext> scheduleResult){
        if(scheduleResult.size()==1){
            return 1;
        }
        Map<Long, Duration> map = scheduleResult.stream().collect(Collectors.toMap(
                s->s.getEquipment().getId(),
                s->{
                    List<IOperationTimeCalculator.OperationTimePlanItem> items = s.getOpertionTimePlan().getPlan();
                    return items.get(items.size()-1).getEndedTime();
                }));
        Map<Long, Duration> sortedMap = MapUtil.sortByValue(map,false);
        int i=1;
        for(Long l : sortedMap.keySet()){
            if(equipment.getId().equals(l)){
                return i;
            }
            i++;
        }
        return i;
    }


    @Override
    public int weight() {
        return 5;
    }

}
