package com.bsn.mes.pro.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.constant.UserConstants;
import com.bsn.common.exception.ServiceException;
import com.bsn.common.utils.data.DataUtil;
import com.bsn.common.utils.data.ExceptionUtil;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.mes.constant.MdItemConst;
import com.bsn.mes.constant.WorkorderStatusEnum;
import com.bsn.mes.md.domain.MdItem;
import com.bsn.mes.md.mapper.MdItemMapper;
import com.bsn.mes.pro.domain.ProCombination;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderDtaRecord;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderSoDirective;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.mapper.ProCombinationMapper;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkOrderSoDirectiveMapper;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkorderMapper;
import com.bsn.mes.pro.service.IProCombinationService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderDtaRecordService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkorderService;
import com.bsn.system.strategy.AutoCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 生产组合单Service业务层处理
 *
 * @author generator
 * @date 2024-02-28
 */
@Service
public class ProCombinationServiceImpl implements IProCombinationService {
    @Autowired
    private ProCombinationMapper proCombinationMapper;

    @Autowired
    private AutoCodeUtil autoCodeUtil;

    @Autowired
    private ProWorkorderMapper proWorkorderMapper;

    @Autowired
    private IProWorkorderService proWorkorderService;

    @Autowired
    private MdItemMapper mdItemMapper;

    @Resource
    private ProWorkOrderSoDirectiveMapper proWorkOrderSoDirectiveMapper;

    @Autowired
    private IProWorkOrderDtaRecordService proWorkOrderDtoRecordService;

    /**
     * 查询生产组合单
     *
     * @param combinationId 生产组合单主键
     * @return 生产组合单
     */
    @Override
    public ProCombination selectProCombinationByCombinationId(Long combinationId) {
        return proCombinationMapper.selectProCombinationByCombinationId(combinationId);
    }

    /**
     * 查询生产组合单列表
     *
     * @param proCombination 生产组合单
     * @return 生产组合单
     */
    @Override
    public List<ProCombination> selectProCombinationList(ProCombination proCombination) {
        return proCombinationMapper.selectProCombinationList(proCombination);
    }


    /**
     * 查询生产组合单列表
     *
     * @param proCombination 生产组合单
     * @return 生产组合单
     */
    @Override
    public List<ProCombination> selectProCombList(ProCombination proCombination) {
        return proCombinationMapper.selectProCombList(proCombination);
    }

    /**
     * 新增生产组合单
     *
     * @param proCombination 生产组合单
     * @return 结果
     */
    @Override
    public int insertProCombination(ProCombination proCombination) {
        proCombination.createAction();
        return proCombinationMapper.insertProCombination(proCombination);
    }

    @Override
    public List<String> batchAddProCombination(List<ProCombination> proCombinations) {
        proCombinations.forEach(s -> {
            ProWorkorder proWorkorder = proWorkorderMapper.selectById(s.getWorkorderId());
            if (!WorkorderStatusEnum.PUBLISHED.getValue().equals(proWorkorder.getStatus())) {
                throw new ServiceException(String.format("%s工单未做刀模板具检查，请先执行刀模板检查功能", proWorkorder.getWorkorderCode()));
            }
        });

        Long[] workorderArr = proCombinations.stream()
                .map(ProCombination::getWorkorderId)
                .toArray(Long[]::new);

        List<String> workorderIds = proCombinationMapper.selectProCombinationInWorkorderGroup(workorderArr);
        List<String> combinationCodes = new ArrayList<>();
        if (proCombinations.get(0) != null && proCombinations.get(0).getCombinationCode() != null && !"".equals(proCombinations.get(0).getCombinationCode()) && workorderIds.size() > 1) {
            throw new RuntimeException("不允许组合！");
        }

        if (workorderIds.size() == 0) {
            for (Long l : workorderArr) {
                workorderIds.add(String.valueOf(l));
            }
        }

        for (int i = 0; i < workorderIds.size(); i++) {
            List<Long> workorderIdArr = Arrays.stream(workorderIds.get(i).split(","))
                    .map(Long::valueOf)
                    .collect(Collectors.toList());

            List<ProCombination> matchedCombinations = proCombinations.stream()
                    .filter(combination -> workorderIdArr.contains(combination.getWorkorderId()))
                    .collect(Collectors.toList());

            String combinationCode = "";
            for (int j = 0; j < matchedCombinations.size(); j++) {
                ProCombination proCombination = matchedCombinations.get(j);
                ProWorkorder proWorkorder = proWorkorderMapper.selectById(proCombination.getWorkorderId());
                if (!WorkorderStatusEnum.PUBLISHED.getValue().equals(proWorkorder.getStatus())) {
                    throw new ServiceException(String.format("%s工单未做刀模板具检查，请先执行刀模板检查功能", proWorkorder.getWorkorderCode()));
                }

                if (proCombination.getCombinationCode() == null) {
                    if (Objects.equals(combinationCode, "")) {
                        combinationCode = autoCodeUtil.genSerialCode(UserConstants.COMBINATION_CODE, "");
                    }
                    proCombination.setCombinationCode(combinationCode);
                    if (!combinationCodes.contains(combinationCode)) {
                        combinationCodes.add(combinationCode);
                    }
                }

                proWorkorderMapper.updateWorkorderStatus(proCombination.getWorkorderId(), WorkorderStatusEnum.COMBINED.getValue());
                if (proCombination.getCombinationId() == null) {
                    this.insertProCombination(proCombination);
                    proWorkorder.setCombinationStartTime(proCombination.getCreateTime());

                    createDtaRecord(proWorkorder);
                } else {
                    this.updateProCombination(proCombination);
                }
            }
        }
        return combinationCodes;

    }

    @Transactional
    @Override
    public List<String> batchAddProCombinationV2(List<ProCombination> proCombinations) {
        proCombinations.forEach(proCombination -> {
            ExceptionUtil.checkTrueThrowException(proCombination != null && proCombination.getCombinationCode() != null && !"".equals(proCombination.getCombinationCode()), "已经组合,不允许操作！");

            ProWorkorder proWorkorder = proWorkorderMapper.selectById(proCombination.getWorkorderId());
            ExceptionUtil.checkTrueThrowException(!WorkorderStatusEnum.PUBLISHED.getValue().equals(proWorkorder.getStatus()), String.format("工单:%s,未做刀模版具检查，请先执行刀模版检查功能", proWorkorder.getWorkorderCode()));
        });


        // 数据分组
        Long[] workorderArr = proCombinations.stream()
                .map(ProCombination::getWorkorderId)
                .toArray(Long[]::new);
        List<String> workorderIdsStr = proCombinationMapper.selectProCombinationInWorkorderGroup(workorderArr);
        Map<Integer, List<List<Long>>> map = new HashMap<>();
        List<List<Long>> IdCollList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(workorderIdsStr)) {
            Set<String> disIdss = new HashSet<>();
            for (String s : workorderIdsStr) {
                List<Long> ids = Arrays.stream(s.split(","))
                        .map(Long::valueOf)
                        .sorted()
                        .collect(Collectors.toList());
                String idsStr = ids.stream().map(x -> DataUtil.getNormalData(x, String.class)).filter(StringUtils::isNotEmpty).collect(Collectors.joining());
                if (disIdss.add(idsStr)) {
                    int size = ids.size();
                    if (size > 1) {
                        IdCollList.add(ids);
                    }
                }
            }
        }

        for (List<Long> allId : IdCollList) {
            int size = allId.size();
            List<List<Long>> lists = map.get(size);
            if (lists == null) {
                lists = new ArrayList<>();
                lists.add(allId);
            } else {
                lists.add(allId);
            }

            map.put(size, lists);
        }


        Set<Long> level1Set1 = proCombinations.stream().map(ProCombination::getWorkorderId).filter(Objects::nonNull).collect(Collectors.toSet());
        Set<Long> level1Set2 = new HashSet<>();
        if (CollectionUtil.isNotEmpty(map)) {
            for (Long l : level1Set1) {
                for (Map.Entry<Integer, List<List<Long>>> entry : map.entrySet()) {
                    List<List<Long>> value = entry.getValue();
                    boolean flag = false;

                    for (List<Long> longs : value) {
                        if (longs.contains(l)) {
                            flag = true;
                            break;
                        }
                    }

                    if (!flag) {
                        level1Set2.add(l);
                    }
                }
            }

            for (Long l : level1Set2) {
                List<List<Long>> lists = map.get(1);
                if (lists == null) {
                    lists = new ArrayList<>();
                }

                List<Long> objects = new ArrayList<>();
                objects.add(l);
                lists.add(objects);
                map.put(1, lists);
            }
        } else {
            for (Long l : level1Set1) {

                List<List<Long>> lists = map.get(1);
                if (lists == null) {
                    lists = new ArrayList<>();
                }

                List<Long> objects = new ArrayList<>();
                objects.add(l);
                lists.add(objects);
                map.put(1, lists);
            }
        }


//        ExceptionUtil.throwExcetion("error");

        List<String> combinationCodes = new ArrayList<>();
        for (Map.Entry<Integer, List<List<Long>>> entry : map.entrySet()) {
            List<List<Long>> combinationList = entry.getValue();
            for (List<Long> combinationIds : combinationList) {
                List<ProCombination> matchedCombinations = proCombinations.stream()
                        .filter(combination -> combinationIds.contains(combination.getWorkorderId()))
                        .collect(Collectors.toList());

                String combinationCode = "";
                Long sort = 1L;
                for (ProCombination proCombination : matchedCombinations) {
                    ProWorkorder proWorkorder = proWorkorderMapper.selectById(proCombination.getWorkorderId());
                    ExceptionUtil.checkTrueThrowException(!WorkorderStatusEnum.PUBLISHED.getValue().equals(proWorkorder.getStatus()), String.format("工单:%s,未做刀模版具检查，请先执行刀模版检查功能", proWorkorder.getWorkorderCode()));


                    if (proCombination.getCombinationCode() == null) {
                        if (Objects.equals(combinationCode, "")) {
                            combinationCode = autoCodeUtil.genSerialCode(UserConstants.COMBINATION_CODE, "");
                        }

                        proCombination.setCombinationCode(combinationCode);
                        if (!combinationCodes.contains(combinationCode)) {
                            combinationCodes.add(combinationCode);
                        }
                    }


                    if (proCombination.getCombinationId() == null) {
                        proCombination.setCombinationSort(sort);
                        this.insertProCombination(proCombination);
                        proWorkorder.setCombinationStartTime(proCombination.getCreateTime());

                        createDtaRecord(proWorkorder);
                        sort++;
                    } else {
                        this.updateProCombination(proCombination);
                    }

                    proWorkorderMapper.updateWorkorderStatus(proCombination.getWorkorderId(), WorkorderStatusEnum.COMBINED.getValue());
                }
            }
        }

        return combinationCodes;
    }


    // 根据item 生成dta记录
    private void createDtaRecord(ProWorkorder proWorkorder) {
        ExceptionUtil.checkTrueThrowException(proWorkorder.getProductId() == null, "当前工单无法找到物料信息");

        Long productId = proWorkorder.getProductId();
        QueryWrapper<MdItem> qw1 = new QueryWrapper<>();
        qw1.eq("item_id", productId);
        List<MdItem> mdItems = mdItemMapper.selectListByQw(qw1);
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(mdItems), "当前工单无法找到物料信息");

        MdItem mdItem = mdItems.get(0);

        if (Objects.equals(mdItem.getDta(), MdItemConst.TURN_ON_DTO_LAYOUT)) {
            QueryWrapper<ProWorkOrderSoDirective> qw2 = new QueryWrapper<>();
            qw2.eq("workorder_id", proWorkorder.getWorkorderId());
            List<ProWorkOrderSoDirective> proWorkOrderSoDirectives = proWorkOrderSoDirectiveMapper.selectListByQw(qw2);
            // 多指令号是固定工单,不需要考虑dta排版
            if (proWorkOrderSoDirectives.size() == 1) {
                ProWorkOrderDtaRecord proWorkOrderDtaRecord = new ProWorkOrderDtaRecord();
                proWorkOrderDtaRecord.setWorkorderId(proWorkorder.getWorkorderId());
                proWorkOrderDtaRecord.setWorkorderCode(proWorkorder.getWorkorderCode());
                proWorkOrderDtaRecord.setSaleCode(proWorkOrderSoDirectives.get(0).getSalesVoucher());
                proWorkOrderDtaRecord.setItemId(mdItem.getItemId());
                proWorkOrderDtaRecord.setSapItemCode(mdItem.getSapItemCode());
                proWorkOrderDtaRecord.setItemName(mdItem.getItemName());
                proWorkOrderDtaRecord.setStartTime(proWorkorder.getCombinationStartTime());
                proWorkOrderDtaRecord.setEndTime(null);
                proWorkOrderDtoRecordService.insertProWorkOrderDtoRecord(proWorkOrderDtaRecord);
            }
        }
    }

    /**
     * 修改生产组合单
     *
     * @param proCombination 生产组合单
     * @return 结果
     */
    @Override
    public int updateProCombination(ProCombination proCombination) {
        return proCombinationMapper.updateProCombination(proCombination);
    }

    /**
     * 批量删除生产组合单
     *
     * @param workorderIds 需要删除的生产组合单主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteProCombinationByWorkorderIds(Long[] workorderIds) {
        List<Long> list = Arrays.asList(workorderIds);
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(list), JSON.toJSONString(list) + ",工单非已组合，不能进行取消组合操作");
        List<ProWorkorder> proWorkorders = proWorkorderMapper.selectListByQw(new QueryWrapper<ProWorkorder>().in("workorder_id", list));

        Set<String> collect = new HashSet<>();
        Set<String> error = new HashSet<>();
        if (CollectionUtil.isNotEmpty(proWorkorders)) {
            for (ProWorkorder proWorkorder : proWorkorders) {
                if (Objects.equals(proWorkorder.getStatus(), WorkorderStatusEnum.COMBINED.getValue())) {
                    proWorkorder.setStatus(WorkorderStatusEnum.PUBLISHED.getValue());
                    proWorkorder.setCombinationCode(null);
                    proWorkorder.setCombinationStartTime(null);
                    proWorkorderService.updateProWorkorder(proWorkorder);
                    collect.add(proWorkorder.getWorkorderCode());
                } else {
                    error.add(proWorkorder.getWorkorderCode());
                }
            }
            ExceptionUtil.checkTrueThrowException(collect.size() != proWorkorders.size(), JSON.toJSONString(error) + ",工单非已组合，不能进行取消组合操作");
        }

        proCombinationMapper.deleteProCombination(new QueryWrapper<ProCombination>().in("workorder_code", collect));
        return 1;
    }

    /**
     * 删除生产组合单信息
     *
     * @param combinationId 生产组合单主键
     * @return 结果
     */
    @Override
    public int deleteProCombinationByCombinationId(Long combinationId) {
        ProCombination proCombination = proCombinationMapper.selectProCombinationByCombinationId(combinationId);
        proWorkorderMapper.updateWorkorderStatus(proCombination.getWorkorderId(), WorkorderStatusEnum.PUBLISHED.getValue());
        return proCombinationMapper.deleteProCombinationByCombinationId(combinationId);
    }
}
