package com.bsn.mes.pro.service.impl.proWorkOrder;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderSoSizeItem;
import com.bsn.mes.pro.domain.vo.proWorkOrder.ProWorkOrderSoSizeItemDynamic;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkOrderSoSizeItemStandard;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkOrderSoSizeItemMapper;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderSoSizeItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * 生产工单尺码Service业务层处理
 *
 * @author zht
 * @date 2024-02-29
 */
@Service
public class ProWorkOrderSoSizeItemServiceImpl implements IProWorkOrderSoSizeItemService {
    @Autowired
    private ProWorkOrderSoSizeItemMapper proWorkOrderSoSizeItemMapper;

    /**
     * 查询生产工单尺码
     *
     * @param workorderSoSizeItemId 生产工单尺码主键
     * @return 生产工单尺码
     */
    @Override
    public ProWorkOrderSoSizeItem selectProWorkOrderSoSizeItemByWorkorderSoSizeItemId(Long workorderSoSizeItemId) {
        return proWorkOrderSoSizeItemMapper.selectProWorkOrderSoSizeItemByWorkorderSoSizeItemId(workorderSoSizeItemId);
    }

    /**
     * 查询生产工单尺码列表
     *
     * @param proWorkOrderSoSizeItem 生产工单尺码
     * @return 生产工单尺码
     */
    @Override
    public List<ProWorkOrderSoSizeItem> selectProWorkOrderSoSizeItemList(ProWorkOrderSoSizeItem proWorkOrderSoSizeItem) {
        return proWorkOrderSoSizeItemMapper.selectProWorkOrderSoSizeItemList(proWorkOrderSoSizeItem);
    }

    @Override
    public ProWorkOrderSoSizeItemDynamic selectProWorkOrderSoSizeItemListDynamic(ProWorkOrderSoSizeItem proWorkOrderSoSizeItem) {
        List<ProWorkOrderSoSizeItem> list = proWorkOrderSoSizeItemMapper.selectProWorkOrderSoSizeItemList(proWorkOrderSoSizeItem);
        ProWorkOrderSoSizeItemDynamic rst = new ProWorkOrderSoSizeItemDynamic();
        rst.setSizeList(list);
        LinkedHashMap<String, String> tempColumns = new LinkedHashMap<String, String>();
        tempColumns.put("standardSizeType", ProWorkOrderSoSizeItemDynamic.ALL_COLUMN.get("standardSizeType"));
        tempColumns.put("quantity", ProWorkOrderSoSizeItemDynamic.ALL_COLUMN.get("quantity"));
        tempColumns.put("lossNum", ProWorkOrderSoSizeItemDynamic.ALL_COLUMN.get("lossNum"));
        tempColumns.put("quantityQualify", "已包装数量");
        tempColumns.put("reportQuantity", "本次包装数量");
        //动态字段
        list.forEach(s -> {
            Map<String, Object> sdata = BeanUtil.beanToMap(s);
            ProWorkOrderSoSizeItemDynamic.ALL_COLUMN.forEach((k, v) -> {
                if (!tempColumns.containsKey(k) && sdata.get(k) != null
                        && !"".equals(sdata.get(k))) {
                    tempColumns.put(k, v);
                }
            });
        });
        rst.setColumns(tempColumns);
        return rst;
    }

    @Override
    public List<Map<String, Object>> selectProWorkOrderSoSizeItemListMap(QueryWrapper<ProWorkOrderSoSizeItem> query) {
        return proWorkOrderSoSizeItemMapper.selectProWorkOrderSoSizeItemListMap(query);
    }

    @Override
    public List<WorkOrderSoSizeItemStandard> selectProWorkOrderSoSizeItemStandardList(QueryWrapper<ProWorkOrderSoSizeItem> query) {
        List<Map<String, Object>> list = proWorkOrderSoSizeItemMapper.selectProWorkOrderSoSizeItemListMap(query);
        List<WorkOrderSoSizeItemStandard> rst = new ArrayList<WorkOrderSoSizeItemStandard>();
        list.forEach(s -> {
            WorkOrderSoSizeItemStandard line = new WorkOrderSoSizeItemStandard();
            line.setIdx((String) s.get("idx"));
            //标准尺码字段
            line.setStandardSizeType((String) s.get("standard_size_type"));
            line.setClientCmd((String) s.get("client_cmd"));
            line.setWorkorderId((Long) s.get("workorder_id"));
            line.setWorkorderSoSizeItemId((Long) s.get("workorder_so_size_item_id"));
            if (s.get("quantity") != null) {
                if (s.get("quantity") instanceof Double) {
                    line.setQuantity(BigDecimal.valueOf((Double) s.get("quantity")));
                } else if (s.get("quantity") instanceof BigDecimal) {
                    line.setQuantity((BigDecimal) s.get("quantity"));
                }

            }

            if (s.get("loss_num") != null) {
                line.setLossNum((BigDecimal) s.get("loss_num"));
            }

            if (line.getStandardSizeType() != null && s.get(line.getStandardSizeType().toLowerCase()) != null) {
                //获取标准尺码值
                line.setStandardSize(s.get(line.getStandardSizeType().toLowerCase()).toString());
            }

            rst.add(line);
        });
        return rst;
    }

    /**
     * 新增生产工单尺码
     *
     * @param proWorkOrderSoSizeItem 生产工单尺码
     * @return 结果
     */
    @Override
    public int insertProWorkOrderSoSizeItem(ProWorkOrderSoSizeItem proWorkOrderSoSizeItem) {
        proWorkOrderSoSizeItem.createAction();
        return proWorkOrderSoSizeItemMapper.insertProWorkOrderSoSizeItem(proWorkOrderSoSizeItem);
    }

    /**
     * 修改生产工单尺码
     *
     * @param proWorkOrderSoSizeItem 生产工单尺码
     * @return 结果
     */
    @Override
    public int updateProWorkOrderSoSizeItem(ProWorkOrderSoSizeItem proWorkOrderSoSizeItem) {
        return proWorkOrderSoSizeItemMapper.updateProWorkOrderSoSizeItem(proWorkOrderSoSizeItem);
    }

    /**
     * 批量删除生产工单尺码
     *
     * @param workorderSoSizeItemIds 需要删除的生产工单尺码主键
     * @return 结果
     */
    @Override
    public int deleteProWorkOrderSoSizeItemByWorkorderSoSizeItemIds(Long[] workorderSoSizeItemIds) {
        return proWorkOrderSoSizeItemMapper.deleteProWorkOrderSoSizeItemByWorkorderSoSizeItemIds(workorderSoSizeItemIds);
    }

    /**
     * 删除生产工单尺码信息
     *
     * @param workorderSoSizeItemId 生产工单尺码主键
     * @return 结果
     */
    @Override
    public int deleteProWorkOrderSoSizeItemByWorkorderSoSizeItemId(Long workorderSoSizeItemId) {
        return proWorkOrderSoSizeItemMapper.deleteProWorkOrderSoSizeItemByWorkorderSoSizeItemId(workorderSoSizeItemId);
    }

    @Override
    public void deleteByWorkorderCode(String workorderCode) {
        QueryWrapper<ProWorkOrderSoSizeItem> proWorkOrderSoSizeItemQueryWrapper = new QueryWrapper<>();
        proWorkOrderSoSizeItemQueryWrapper.eq("workorder_code", workorderCode);
        proWorkOrderSoSizeItemMapper.deleteProWorkOrderSoSizeItem(proWorkOrderSoSizeItemQueryWrapper);
    }
}
