package com.bsn.mes.pro.service.impl.task;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.text.StrJoiner;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.core.domain.entity.SysUser;
import com.bsn.common.exception.ServiceException;
import com.bsn.common.utils.SecurityUtils;
import com.bsn.common.utils.data.DateUtils;
import com.bsn.mes.md.domain.MdItem;
import com.bsn.mes.md.mapper.MdItemMapper;
import com.bsn.mes.md.service.IMdItemService;
import com.bsn.mes.pro.domain.ProFeedback;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.task.ProPackagePrintRecord;
import com.bsn.mes.pro.domain.task.ProTaskWorkunit;
import com.bsn.mes.pro.domain.vo.ProPackagePrintRecordQuery;
import com.bsn.mes.pro.domain.vo.proWorkOrder.BuildPackageParams2;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkorderPackage;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkorderPackagePrint;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkorderPackageResult;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageAddParams;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageQueryParams;
import com.bsn.mes.pro.domain.vo.task.RePackagePrintParams;
import com.bsn.mes.pro.mapper.task.ProPackagePrintRecordMapper;
import com.bsn.mes.pro.service.IProFeedbackService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderSoDirectiveService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderSoSizeItemService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkorderService;
import com.bsn.mes.pro.service.task.IProPackagePrintRecordService;
import com.bsn.system.service.ISysUserService;
import com.bsn.system.strategy.AutoCodeUtil;
import com.google.common.base.Joiner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 报工包装标签打印记录表Service业务层处理
 *
 * @author generator
 * @date 2024-03-19
 */
@Service
public class ProPackagePrintRecordServiceImpl implements IProPackagePrintRecordService {
    @Autowired
    private ProPackagePrintRecordMapper proPackagePrintRecordMapper;
    @Autowired
    private IProWorkorderService proWorkorderService;
    @Autowired
    private IMdItemService mdItemService;
    @Autowired
    private AutoCodeUtil autoCodeUtil;
    @Autowired
    private IProWorkOrderSoDirectiveService proWorkOrderSoDirectiveService;
    @Autowired
    private IProWorkOrderSoSizeItemService proWorkOrderSoSizeItemService;
    @Autowired
    private IProFeedbackService proFeedbackService;
    @Autowired
    private MdItemMapper mdItemMapper;
    @Autowired
    private ISysUserService sysUserService;


    /**
     * 查询报工包装标签打印记录表
     *
     * @param recordId 报工包装标签打印记录表主键
     * @return 报工包装标签打印记录表
     */
    @Override
    public ProPackagePrintRecord selectProPackagePrintRecordByRecordId(Long recordId) {
        return proPackagePrintRecordMapper.selectProPackagePrintRecordByRecordId(recordId);
    }

    @Override
    public ProPackagePrintRecord selectProPackagePrintRecordByQrcode(String qrcode) {
        QueryWrapper<ProPackagePrintRecord> query = new QueryWrapper<ProPackagePrintRecord>();
        query.eq("qrcode", qrcode);
        List<ProPackagePrintRecord> list = proPackagePrintRecordMapper.selectProPackagePrintRecordListByQw(query);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public ProPackagePrintRecord selectProPackagePrintRecordByBarcode(String barcode) {
        QueryWrapper<ProPackagePrintRecord> query = new QueryWrapper<ProPackagePrintRecord>();
        query.eq("barcode", barcode);
        List<ProPackagePrintRecord> list = proPackagePrintRecordMapper.selectProPackagePrintRecordListByQw(query);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 查询报工包装标签打印记录表列表
     *
     * @param proFdPackagePrintRecord 报工包装标签打印记录表
     * @return 报工包装标签打印记录表
     */
    @Override
    public List<ProPackagePrintRecord> selectProPackagePrintRecordList(ProPackagePrintRecordQuery proFdPackagePrintRecord) {
        return proPackagePrintRecordMapper.selectProPackagePrintRecordList(proFdPackagePrintRecord);
    }

    @Override
    public List<ProPackagePrintRecord> selectProPackagePrintRecordList(QueryWrapper<ProPackagePrintRecord> query) {
        return proPackagePrintRecordMapper.selectProPackagePrintRecordListByQw(query);
    }

    /**
     * 新增报工包装标签打印记录表
     *
     * @param proFdPackagePrintRecord 报工包装标签打印记录表
     * @return 结果
     */
    @Override
    public int insertProPackagePrintRecord(ProPackagePrintRecord proFdPackagePrintRecord) {
        if(proFdPackagePrintRecord.getQrcode().startsWith(BL)){
            proFdPackagePrintRecord.setBarcode(autoCodeUtil.genSerialCode(BL_PACK_CODE, ""));
        }else{
            proFdPackagePrintRecord.setBarcode(autoCodeUtil.genSerialCode(PL_PACK_CODE, ""));
        }
        proFdPackagePrintRecord.setCreateBy(SecurityUtils.getUsername());
        proFdPackagePrintRecord.setCreateTime(DateUtils.getNowDate());
        return proPackagePrintRecordMapper.insertProPackagePrintRecord(proFdPackagePrintRecord);
    }

    @Override
    public ProPackagePrintRecord insertProPackagePrintRecordByFeedback(ProTaskWorkunit taskWorkunit, ProWorkorder workorder,
                                                                       FeedbackPackageAddParams feedbackAddParams, Integer allReportNum, BigDecimal netWeight) {
        String qrcode = feedbackAddParams.getReportData().getQrcode();
        ProPackagePrintRecord tempPackage = this.selectProPackagePrintRecordByQrcode(qrcode);
        String checkNo = feedbackAddParams.getCheckBy();
        SysUser sysUser = sysUserService.selectUserByUserName(checkNo);
        //混合包装，存在单个标签多次报工。不存在标签保存记录
        if (tempPackage != null) {
            tempPackage.setNetWeight(netWeight);
            if(sysUser!=null){
                tempPackage.setCheckBy(sysUser.getNickName());
            }
            this.updateProPackagePrintRecord(tempPackage);
            return tempPackage;
        }
        ProPackagePrintRecord proFdPackagePrintRecord = new ProPackagePrintRecord();
        WorkorderPackageResult workorderPackageResult = null;
        if(!qrcode.startsWith(SL)) {//非超报标签
            BuildPackageParams2 buildPackageParams2 = new BuildPackageParams2();
            buildPackageParams2.setWorkorderId(workorder.getWorkorderId());
            buildPackageParams2.setQuantity(workorder.getQuantity().intValue());
            buildPackageParams2.setEncasementNum(workorder.getUsageEncasementNum());
            buildPackageParams2.setPackNum(workorder.getUsagePackNum());
            buildPackageParams2.setMergePack(workorder.getMergePack() == 1);
            workorderPackageResult = proWorkorderService.buildPackagePrintInfo(buildPackageParams2);
            WorkorderPackage workorderPackage = workorderPackageResult.getPackageList().stream().filter(s -> s.getQrcode().equals(qrcode)).findFirst()
                    .orElseThrow(() -> new ServiceException(String.format("未找到打印包装标签:%s", qrcode)));
            BeanUtil.copyProperties(workorderPackage, proFdPackagePrintRecord);
            if (workorderPackage.getPackageNum() + workorderPackage.getLossNum() < allReportNum) {
                throw new ServiceException("报工合格数量不能大于包装数量");
            }
            ProPackagePrintRecordServiceImpl.initSizeIds(workorderPackage, proFdPackagePrintRecord);
        }else if(qrcode.startsWith(SL)){
            proFdPackagePrintRecord.setIsSurplus(1);
            proFdPackagePrintRecord.setLossNum(0);
            proFdPackagePrintRecord.setSheetCount(1);
            proFdPackagePrintRecord.setTotalCount(1);
            proFdPackagePrintRecord.setPackageNum(feedbackAddParams.getReportData().getReportQuantity().intValue());
        }
        proFdPackagePrintRecord.setWorkorderId(workorder.getWorkorderId());
        proFdPackagePrintRecord.setTaskWorkunitId(taskWorkunit.getTaskWorkunitId());
        proFdPackagePrintRecord.setQrcode(qrcode);
        proFdPackagePrintRecord.setNetWeight(netWeight);
        proFdPackagePrintRecord.setInputBy(SecurityUtils.getUsername());
        if(sysUser != null){
            proFdPackagePrintRecord.setCheckBy(sysUser.getNickName());
        }
        this.insertProPackagePrintRecord(proFdPackagePrintRecord);

        //如果紧接装箱标签一并保存
        if(workorderPackageResult!=null){
            java.util.Iterator<WorkorderPackage> iterator = workorderPackageResult.getPackageList().iterator();
            while(iterator.hasNext()){
                WorkorderPackage temp = iterator.next();
                if(temp.getQrcode().equals(qrcode)){
                    if(iterator.hasNext()){
                        WorkorderPackage blTemp = iterator.next();
                        if(blTemp.getQrcode().startsWith("BL")){
                            ProPackagePrintRecord proFdPackagePrintRecordBl = new ProPackagePrintRecord();
                            BeanUtil.copyProperties(blTemp, proFdPackagePrintRecordBl);
                            proFdPackagePrintRecordBl.setWorkorderId(workorder.getWorkorderId());
                            proFdPackagePrintRecordBl.setTaskWorkunitId(taskWorkunit.getTaskWorkunitId());
                            proFdPackagePrintRecordBl.setQrcode(blTemp.getQrcode());
                            proFdPackagePrintRecordBl.setInputBy(SecurityUtils.getUsername());
                            ProPackagePrintRecordServiceImpl.initSizeIds(blTemp, proFdPackagePrintRecordBl);
                            this.insertProPackagePrintRecord(proFdPackagePrintRecordBl);
                        }
                        break;
                    }
                }
            }
        }
        return proFdPackagePrintRecord;
    }

    /**
     * 修改报工包装标签打印记录表
     *
     * @param proFdPackagePrintRecord 报工包装标签打印记录表
     * @return 结果
     */
    @Override
    public int updateProPackagePrintRecord(ProPackagePrintRecord proFdPackagePrintRecord) {
        return proPackagePrintRecordMapper.updateProPackagePrintRecord(proFdPackagePrintRecord);
    }

    /**
     * 批量删除报工包装标签打印记录表
     *
     * @param recordIds 需要删除的报工包装标签打印记录表主键
     * @return 结果
     */
    @Override
    public int deleteProPackagePrintRecordByRecordIds(Long[] recordIds) {
        return proPackagePrintRecordMapper.deleteProPackagePrintRecordByRecordIds(recordIds);
    }

    /**
     * 删除报工包装标签打印记录表信息
     *
     * @param recordId 报工包装标签打印记录表主键
     * @return 结果
     */
    @Override
    public int deleteProPackagePrintRecordByRecordId(Long recordId) {
        return proPackagePrintRecordMapper.deleteProPackagePrintRecordByRecordId(recordId);
    }

    @Override
    public int count(QueryWrapper<ProPackagePrintRecord> query) {
        return proPackagePrintRecordMapper.count(query);
    }

    @Override
    public void batchUpdate(List<ProPackagePrintRecord> list) {
        for (ProPackagePrintRecord record : list) {
            ProPackagePrintRecord dbRecord = proPackagePrintRecordMapper.selectProPackagePrintRecordByRecordId(record.getRecordId());
            dbRecord.setNetWeight(record.getNetWeight());
            dbRecord.setRemark(record.getRemark());
            proPackagePrintRecordMapper.updateProPackagePrintRecord(dbRecord);
        }
    }

    @Override
    public List<WorkorderPackagePrint> recordPackagePrintInfo(Long workorderId, List<ProPackagePrintRecord> tempList) {
        ProWorkorder proWorkorder = proWorkorderService.selectWorkorderById(workorderId);
        MdItem item = mdItemService.selectMdItemById(proWorkorder.getProductId());
        List<WorkorderPackagePrint> rst = this.getPackagePrintInfo(tempList, workorderId, false);
        //打印次数+1
        QueryWrapper<ProPackagePrintRecord> query = new QueryWrapper<ProPackagePrintRecord>();
        query.in("record_id", tempList.stream().map(s -> s.getRecordId()).collect(Collectors.toList()));
        this.printTimeAccumulation(query);

        return rst;
    }

    @Override
    public void printTimeAccumulation(QueryWrapper<ProPackagePrintRecord> query) {
        proPackagePrintRecordMapper.printTimeAccumulation(query);
    }

    @Override
    public List<WorkorderPackagePrint> getPackagePrintInfo(ProPackagePrintRecord packagePrintRecord, Long orderId, boolean isPrintEncasement) {
        List<ProPackagePrintRecord> list = new ArrayList<ProPackagePrintRecord>();
        list.add(packagePrintRecord);
        return this.getPackagePrintInfo(list, orderId, isPrintEncasement);
    }

    private WorkorderPackagePrint convert(WorkorderPackage packageInfo, WorkorderPackageResult packageResult) {
        WorkorderPackagePrint rst = new WorkorderPackagePrint();
        BeanUtil.copyProperties(packageInfo, rst);
        if (packageInfo.getLossNum() != 0) {
            rst.setPackageNum(String.format("%s+%s", packageInfo.getPackageNum(), packageInfo.getLossNum()));
        } else {
            rst.setPackageNum(packageInfo.getPackageNum() + "");
        }
        //查询报工
        QueryWrapper<ProFeedback> feedbackQuery = new QueryWrapper<ProFeedback>();
        if (packageInfo.getQrcode().startsWith("BL")) {
            List<String> qrcodes = packageInfo.getPackageList().stream().map(s -> s.getQrcode()).collect(Collectors.toList());
            feedbackQuery.in("qrcode", qrcodes);
        } else {
            feedbackQuery.eq("qrcode", packageInfo.getQrcode());
        }
        List<ProFeedback> feedbackList = proFeedbackService.selectProFeedbackList(feedbackQuery);
        Integer reportedNum = feedbackList.stream().mapToInt(s -> s.getQuantityQualify().intValue()).sum();
        if (packageInfo.getLossNum() != 0) {
            rst.setReportNum(String.format("%s+%s", reportedNum, packageInfo.getLossNum()));
        } else {
            rst.setReportNum(reportedNum + "");
        }
        if(feedbackList.size()==0){
            rst.setFeedbackTime(DateUtils.getNowDate());
        }
        if (packageInfo.getQrcode().startsWith("BL")) {
            int size = feedbackList.size();
            size = size - 1;
            ProFeedback feedback = feedbackList.get(size);
            rst.setFeedbackTime(feedback.getFeedbackTime());
        } else {
            for (ProFeedback feedback : feedbackList) {
                rst.setFeedbackTime(feedback.getFeedbackTime());
            }
        }


        if (packageInfo.getDistributes() != null && !packageInfo.getDistributes().isEmpty()) {
            StrJoiner tempStr = StrJoiner.of(",");
            AtomicReference<String> tempClientCmd = new AtomicReference<String>();
            for (WorkorderPackage.QuantityDistribute quantityDistribute : packageInfo.getDistributes()) {
                if (quantityDistribute.getDirectiveId() != null) {
                    packageResult.getDirectiveList().stream().filter(s -> s.getWorkorderSoDirectiveId().equals(quantityDistribute.getDirectiveId())).findFirst().ifPresent(s -> {
                        tempStr.append(s.getClientCmd());
                    });
                }
                if (quantityDistribute.getSizeId() != null) {
                    packageResult.getSizeList().stream().filter(s -> s.getWorkorderSoSizeItemId().equals(quantityDistribute.getSizeId())).findFirst().ifPresent(s -> {
                        String standardSizeType = s.getStandardSizeType();
                        if (standardSizeType != null) {
                            String val = (String) BeanUtil.getFieldValue(s, standardSizeType.toLowerCase());
                            tempStr.append(val);
                            tempClientCmd.set(s.getClientCmd());
                        }
                    });
                }
            }
            if (packageInfo.getDistributes().get(0).getDirectiveId() != null) {
                rst.setClientCmd(tempStr.toString());
            }
            if (packageInfo.getDistributes().get(0).getSizeId() != null) {
                rst.setStandardSize(tempStr.toString());
                rst.setClientCmd(tempClientCmd.get());
            }
        } else {
            packageResult.getDirectiveList().stream().filter(s -> s.getWorkorderSoDirectiveId().equals(packageInfo.getWorkorderSoDirectiveId())).findFirst().ifPresent(s -> {
                rst.setClientCmd(s.getClientCmd());
            });
            packageResult.getSizeList().stream().filter(s -> s.getWorkorderSoSizeItemId().equals(packageInfo.getWorkorderSoSizeItemId())).findFirst().ifPresent(s -> {
                String standardSizeType = s.getStandardSizeType();
                if (standardSizeType != null) {
                    String val = (String) BeanUtil.getFieldValue(s, standardSizeType.toLowerCase());
                    rst.setStandardSize(val);
                    rst.setClientCmd(s.getClientCmd());
                }
            });
        }
        return rst;
    }

    /*
     * 转换包装标签打印信息
     * @param packagePrintRecordList 包装标签记录
     * @param orderId 工单ID
     * @param isPrintEncasement 是否打印装箱标签
     * @return
     */
    private List<WorkorderPackagePrint> getPackagePrintInfo(List<ProPackagePrintRecord> packagePrintRecordList, Long orderId, boolean isPrintEncasement) {
        ProWorkorder workorder = proWorkorderService.selectWorkorderById(orderId);
        MdItem mdItem = mdItemService.selectMdItemById(workorder.getProductId());
        String sapItemCode = mdItem.getSapItemCode();
        String substring = sapItemCode.substring(10);
        mdItem.setSapItemCode(substring);
        List<WorkorderPackagePrint> rst = new ArrayList<WorkorderPackagePrint>();
        BuildPackageParams2 buildPackageParams = new BuildPackageParams2();
        buildPackageParams.setWorkorderId(orderId);
        buildPackageParams.setEncasementNum(workorder.getUsageEncasementNum());
        buildPackageParams.setPackNum(workorder.getUsagePackNum());
        buildPackageParams.setMergePack(workorder.getMergePack()==1);
        WorkorderPackageResult packageResult = proWorkorderService.buildPackagePrintInfo(buildPackageParams);
        for (ProPackagePrintRecord packagePrintRecord : packagePrintRecordList) {
            if(packagePrintRecord.getQrcode().startsWith(IProPackagePrintRecordService.SL)){
                WorkorderPackagePrint currPackagePrintInfo = this.convert(packageResult.getPackageList().get(0), packageResult);
                currPackagePrintInfo.setValues(packagePrintRecord, workorder, mdItem);
                currPackagePrintInfo.setPackageNum(packagePrintRecord.getPackageNum()+"");
                rst.add(currPackagePrintInfo);
            }else {
                for (int i = 0; i < packageResult.getPackageList().size(); i++) {
                    WorkorderPackage temp = packageResult.getPackageList().get(i);
                    if (temp.getQrcode().equals(packagePrintRecord.getQrcode())) {
                        WorkorderPackagePrint currPackagePrintInfo = this.convert(temp, packageResult);
                        currPackagePrintInfo.setValues(packagePrintRecord, workorder, mdItem);
                        rst.add(currPackagePrintInfo);
                        if (isPrintEncasement && packageResult.getPackageList().size() > i + 1) {
                            WorkorderPackage nextPackage = packageResult.getPackageList().get(i + 1);
                            if (nextPackage.getQrcode().startsWith("BL")) {
                                ProPackagePrintRecord encasementPrintRecord = this.selectProPackagePrintRecordByQrcode(nextPackage.getQrcode());
                                WorkorderPackagePrint packagePrintInfo2 = this.convert(nextPackage, packageResult);
                                packagePrintInfo2.setValues(encasementPrintRecord, workorder, mdItem);
                                rst.add(packagePrintInfo2);
                            }
                        }
                        break;
                    }
                }
            }
        }
        return rst;
    }

    public static void initSizeIds(WorkorderPackage s, ProPackagePrintRecord packagePrintRecord){
        if(s.getQrcode().startsWith("BL")){
            Set<String> sizeUnique = new HashSet<String>();
            StrJoiner j2 = StrJoiner.of(",");
            s.getPackageList().forEach(t->{
                if(t.getDistributes()!=null&&t.getDistributes().size()>0){
                    t.getDistributes().stream().filter(t2 -> t2.getSizeId() != null).forEach(t2->{
                        if(!sizeUnique.contains(t2.getSizeId()+"")){
                            j2.append(t2.getSizeId()+"");
                        }
                        sizeUnique.add(t2.getSizeId()+"");
                    });
                }else if(t.getWorkorderSoSizeItemId()!=null){
                    if(!sizeUnique.contains(t.getWorkorderSoSizeItemId() + "")) {
                        j2.append(t.getWorkorderSoSizeItemId() + "");
                    }
                    sizeUnique.add(t.getWorkorderSoSizeItemId()+"");
                }
            });
            if(j2.toString().length()>0){
                packagePrintRecord.setWorkorderSoSizeItemIds(j2.toString());
            }
        }else {
            StrJoiner j = StrJoiner.of(",");
            if (s.getDistributes() != null && s.getDistributes().size() > 0) {
                s.getDistributes().stream().filter(t -> t.getSizeId() != null).forEach(t -> {
                    j.append(t.getSizeId() + "");
                });
            }
            if (j.toString().length() > 0) {
                packagePrintRecord.setWorkorderSoSizeItemIds(j.toString());
            }
        }
    }

    @Override
    public List<WorkorderPackagePrint> rePackageInfoPrint(RePackagePrintParams query) {
        ProPackagePrintRecord record = this.selectProPackagePrintRecordByQrcode(query.getQrcode());
        return this.getPackagePrintInfo(record, query.getWorkorderId(), true);
    }
}
