package com.bsn.mes.pro.service.impl.task;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.utils.data.DataUtil;
import com.bsn.common.utils.data.ExceptionUtil;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.mes.constant.AssistProcessConst;
import com.bsn.mes.constant.TaskConst;
import com.bsn.mes.constant.TaskStatusEnum;
import com.bsn.mes.constant.TaskWorkunitStatusEnum;
import com.bsn.mes.pro.domain.ProFeedback;
import com.bsn.mes.pro.domain.ProProcess;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderProcess;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.task.ProTask;
import com.bsn.mes.pro.domain.task.ProTaskAssistProcess;
import com.bsn.mes.pro.domain.task.ProTaskWorkunit;
import com.bsn.mes.pro.domain.vo.ProTaskVo;
import com.bsn.mes.pro.domain.vo.ProWorkOrderProcessVo;
import com.bsn.mes.pro.mapper.ProFeedbackMapper;
import com.bsn.mes.pro.mapper.ProProcessMapper;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkorderMapper;
import com.bsn.mes.pro.mapper.task.ProTaskAssistProcessMapper;
import com.bsn.mes.pro.mapper.task.ProTaskMapper;
import com.bsn.mes.pro.mapper.task.ProTaskWorkunitMapper;
import com.bsn.mes.pro.service.IProFeedbackService;
import com.bsn.mes.pro.service.IProStartWorkService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderProcessService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkorderService;
import com.bsn.mes.pro.service.task.IProTaskAssistProcessService;
import com.bsn.mes.pro.service.task.IProTaskService;
import com.bsn.mes.pro.service.task.IProTaskWorkunitService;
import com.bsn.mes.sap.SapService;
import com.bsn.mes.sap.execute.SapServiceAssistProcess;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import static com.bsn.mes.constant.TaskWorkunitStatusEnum.BEGINNING;
import static com.bsn.mes.constant.TaskWorkunitStatusEnum.FINISHED;

/**
 * 工序任务外协关系Service业务层处理
 *
 * @author generator
 * @date 2024-03-29
 */
@Slf4j
@Service
public class ProTaskAssistProcessServiceImpl implements IProTaskAssistProcessService {

    @Autowired
    private ProTaskAssistProcessMapper proTaskAssistProcessMapper;

    @Resource
    private ProFeedbackMapper proFeedbackMapper;
    @Resource
    private ProWorkorderMapper proWorkorderMapper;
    @Autowired
    private IProWorkorderService proWorkorderService;

    @Autowired
    private IProTaskWorkunitService proTaskWorkunitService;

    @Resource
    private ProTaskWorkunitMapper proTaskWorkunitMapper;

    @Autowired
    private IProFeedbackService proFeedbackService;

    @Autowired
    private IProStartWorkService proStartWorkService;
    @Autowired
    private IProTaskService proTaskService;
    @Resource
    private ProTaskMapper proTaskMapper;

    @Autowired
    private IProWorkOrderProcessService proWorkOrderProcessService;

    @Resource
    private ProProcessMapper proProcessMapper;


    @Resource
    private SapService sapService;

    /**
     * 查询工序任务外协关系
     *
     * @param id 工序任务外协关系主键
     * @return 工序任务外协关系
     */
    @Override
    public ProTaskAssistProcess selectProTaskAssistProcessById(Long id) {
        return proTaskAssistProcessMapper.selectProTaskAssistProcessById(id);
    }

    @Override
    public Boolean queryAssistProcessIsCompleted(String code) {
        return proTaskAssistProcessMapper.selectTaskAssistProcessByTwCode(code) != null;
    }

    /**
     * 查询工序任务外协关系列表
     *
     * @param proTaskAssistProcesses 工序任务外协关系
     * @return 工序任务外协关系
     */
    @Override
    public List<ProTaskAssistProcess> listUnreportedFeedback(ProTaskAssistProcess proTaskAssistProcesses) {
        return this.selectListByObj(proTaskAssistProcesses).stream().filter(x -> x.getAssistProcessCode() != null && this.queryAssistProcessIsCompleted(x.getAssistProcessCode())).collect(Collectors.toList());
    }


    /**
     * 查询工序任务外协关系列表
     *
     * @param proTaskAssistProcesses 工序任务外协关系
     * @return 工序任务外协关系
     */
    @Override
    public List<ProTaskAssistProcess> selectListByObj(ProTaskAssistProcess proTaskAssistProcesses) {
        QueryWrapper<ProTaskAssistProcess> proTaskAssistProcessQueryWrapper = new QueryWrapper<>();
        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getAssistProcessCode()), "assist_process_code", proTaskAssistProcesses.getAssistProcessCode());
        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getAssistProcessItemCode()), "assist_process_item_code", proTaskAssistProcesses.getAssistProcessItemCode());
        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getProcessId()), "process_id", proTaskAssistProcesses.getProcessId());
        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getTaskWorkunitId()), "task_workunit_id", proTaskAssistProcesses.getTaskWorkunitId());

        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getProcessName()), "process_name", proTaskAssistProcesses.getProcessName());
        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getWorkorderId()), "workorder_id", proTaskAssistProcesses.getWorkorderId());
        proTaskAssistProcessQueryWrapper.eq(StringUtils.isNotEmpty(proTaskAssistProcesses.getWorkorderCode()), "workorder_code", proTaskAssistProcesses.getWorkorderCode());
        proTaskAssistProcessQueryWrapper.eq("delete_state", 0);
        List<ProTaskAssistProcess> proTaskAssistProcessList = proTaskAssistProcessMapper.selectListByQw(proTaskAssistProcessQueryWrapper);
        for (ProTaskAssistProcess proTaskAssistProcess : proTaskAssistProcessList) {
            Long taskWorkunitId = proTaskAssistProcess.getTaskWorkunitId();
            Long sum = proFeedbackMapper.selectListByQw(new QueryWrapper<ProFeedback>().eq("task_workunit_id", taskWorkunitId)).stream().filter(x -> x.getQuantityFeedback() != null).collect(Collectors.summarizingLong(y -> y.getQuantityFeedback().longValue())).getSum();
            proTaskAssistProcess.setQuantityFeedback(sum);
        }
        return proTaskAssistProcessList;
    }


    @Override
    public ProTaskAssistProcess selectBaseByObj(ProTaskAssistProcess proTaskAssistProcesses) {
        List<ProTaskAssistProcess> proTaskAssistProcessList = this.selectListByObj(proTaskAssistProcesses);
        return CollectionUtil.isEmpty(proTaskAssistProcessList) ? null : proTaskAssistProcessList.get(0);
    }


    /**
     * 新增工序任务外协关系
     *
     * @param proTaskAssistProcess 工序任务外协关系
     * @return 结果
     */
    @Override
    public int insertProTaskAssistProcess(ProTaskAssistProcess proTaskAssistProcess) {
        proTaskAssistProcess.createAction();
        return proTaskAssistProcessMapper.insertProTaskAssistprocess(proTaskAssistProcess);
    }


    /**
     * 修改工序任务外协关系
     *
     * @param proTaskAssistProcess 工序任务外协关系
     * @return 结果
     */
    @Override
    public int updateProTaskAssistprocess(ProTaskAssistProcess proTaskAssistProcess) {
        proTaskAssistProcess.updateAction();
        return proTaskAssistProcessMapper.updateProTaskAssistProcess(proTaskAssistProcess);
    }

    private void checkFeedBack(Long taskWorkunitId) {
        List<ProFeedback> proFeedbacks = proFeedbackMapper.selectListByQw(new QueryWrapper<ProFeedback>().eq("task_workunit_id", taskWorkunitId));
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(proFeedbacks), "已经报工,不可以进行变更");

    }

    /**
     * 修改工序任务外协关系
     *
     * @param updateAssistProcess 工序任务外协关系
     * @return 结果
     */
    @Transactional
    @Override
    public int updateObj(ProTaskAssistProcess updateAssistProcess) {
        ProTaskAssistProcess qo = new ProTaskAssistProcess();
        qo.setAssistProcessCode(updateAssistProcess.getAssistProcessCode());
        qo.setAssistProcessItemCode(updateAssistProcess.getAssistProcessItemCode());
        ProTaskAssistProcess assistProcess = this.selectBaseByObj(qo);
        ExceptionUtil.checkTrueThrowException(assistProcess == null, "外协记录不存在");
        ExceptionUtil.checkTrueThrowException(assistProcess.getCloseType() == 1, "外协任务已经关闭,不可以更新外协数量");
        checkFeedBack(assistProcess.getTaskWorkunitId());

        Long oldQuantity = assistProcess.getQuantity();
        Long updateQuantity = updateAssistProcess.getQuantity();
        assistProcess.setApiFlag(true);
        if (oldQuantity - updateQuantity == 0) {
            this.updateProTaskAssistProcess(assistProcess);
            return 1;
        }


        Long taskId = assistProcess.getTaskId();
        ProTask proTask = proTaskService.selectBaseById(taskId);
        if (proTask == null) {
            this.updateProTaskAssistProcess(assistProcess);
            return 1;
        }

        ExceptionUtil.checkTrueThrowException(updateQuantity.intValue() > proTask.getQuantity().intValue(), "编排数量大于工序任务数量,无法更新");
        checkAssistQuantity(proTask, updateAssistProcess);


        ProWorkOrderProcess proWorkOrderProcess = new ProWorkOrderProcess();
        proWorkOrderProcess.setProcessId(assistProcess.getProcessId());
        proWorkOrderProcess.setWorkorderId(assistProcess.getWorkorderId());
        List<ProWorkOrderProcessVo> proWorkOrderProcessVos = proWorkOrderProcessService.selectProWorkOrderProcessList(proWorkOrderProcess);
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(proWorkOrderProcessVos), "工序任务不存在");
        ProWorkOrderProcessVo processVo = proWorkOrderProcessVos.get(0);
        proTaskWorkunitService.deleteProTaskWorkunitByTaskWorkunitId(assistProcess.getTaskWorkunitId());

        assistProcess.setWorkorderProcessId(processVo.getWorkorderProcessId());
        assistProcess.setTaskId(proTask.getTaskId());
        assistProcess.setTaskCode(proTask.getTaskCode());
        ProTaskVo proTaskVo = proTask.deepCopyObj(ProTaskVo.class);

        ProTaskWorkunit taskWorkunit = new ProTaskWorkunit();
        taskWorkunit.setQuantity(BigDecimal.valueOf(assistProcess.getQuantity()));
        taskWorkunit.setWorkunitId(AssistProcessConst.WORKUNIT_ID);
        taskWorkunit.setWorkstationId(AssistProcessConst.WORKSTATION_ID);
        taskWorkunit.setWorkstationCode(AssistProcessConst.WORKSTATION_CODE);
        proTaskVo.setSplitTaskWorkunit(taskWorkunit);
//        proTaskVo.setTaskWorkunitId(assistProcess.getTaskWorkunitId());
        taskWorkunit = proTaskService.splitTask(proTaskVo, TaskConst.ASSIST_TASK);
        assistProcess.setTaskWorkunitId(taskWorkunit.getTaskWorkunitId());
        assistProcess.setWorkunitId(taskWorkunit.getWorkunitId());
        this.updateProTaskAssistProcess(assistProcess);
        return 1;
    }


    /**
     * 修改工序任务外协关系
     *
     * @param proTaskAssistprocess 工序任务外协关系
     * @return 结果
     */
    @Override
    public int updateProTaskAssistProcess(ProTaskAssistProcess proTaskAssistprocess) {
        proTaskAssistprocess.updateAction();
        return proTaskAssistProcessMapper.updateProTaskAssistProcess(proTaskAssistprocess);
    }


    /**
     * 批量删除工序任务外协关系
     *
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteObj(ProTaskAssistProcess updateAssistProcess) {
        ProTaskAssistProcess qo = new ProTaskAssistProcess();
        qo.setAssistProcessCode(updateAssistProcess.getAssistProcessCode());
        qo.setAssistProcessItemCode(updateAssistProcess.getAssistProcessItemCode());
        ProTaskAssistProcess assistProcess = this.selectBaseByObj(qo);

        ExceptionUtil.checkTrueThrowException(assistProcess == null, "外协记录不存在");
        ExceptionUtil.checkTrueThrowException(assistProcess.getCloseType() == 1, "外协任务已经关闭,不可以更新外协数量");
        checkFeedBack(assistProcess.getTaskWorkunitId());

        ProTaskWorkunit mainTaskWorkunit = proTaskWorkunitService.selectMainObjById(updateAssistProcess.getTaskId());
        ProTaskWorkunit proTaskWorkunit = proTaskWorkunitService.selectProTaskWorkunitByTaskWorkunitId(updateAssistProcess.getTaskWorkunitId());
        mainTaskWorkunit.setQuantity(mainTaskWorkunit.getQuantity().add(proTaskWorkunit.getQuantity()));
        proTaskWorkunitService.updateProTaskWorkunit(mainTaskWorkunit);
        proTaskWorkunitService.deleteProTaskWorkunitByTaskWorkunitId(proTaskWorkunit.getTaskWorkunitId());
        this.deleteProTaskAssistProcessById(assistProcess.getId());
        return 1;
    }

    /**
     * 批量删除工序任务外协关系
     *
     * @param ids 需要删除的工序任务外协关系主键
     * @return 结果
     */
    @Override
    public int deleteProTaskAssistProcessByIds(Long[] ids) {
        return proTaskAssistProcessMapper.deleteProTaskAssistProcessByIds(ids);
    }

    /**
     * 删除工序任务外协关系信息
     *
     * @param id 工序任务外协关系主键
     * @return 结果
     */
    @Override
    public int deleteProTaskAssistProcessById(Long id) {
        return proTaskAssistProcessMapper.deleteProTaskAssistProcessById(id);
    }


    @Transactional
    @Override
    public int feedBack(ProFeedback feedback) {
        Long taskWorkunitId = feedback.getTaskWorkunitId();
        ProTaskAssistProcess proTaskAssistProcess = this.selectBaseByObj(new ProTaskAssistProcess(taskWorkunitId));
        ExceptionUtil.checkTrueThrowException(StringUtils.isEmpty(proTaskAssistProcess), "外协记录不存在");

        ProTaskWorkunit taskWorkunit = proTaskWorkunitService.selectProTaskWorkunitByTaskWorkunitId(taskWorkunitId);
        ExceptionUtil.checkTrueThrowException(taskWorkunit == null, "排产任务不存在");
        ExceptionUtil.checkTrueThrowException(!taskWorkunit.getStatus().equals(TaskStatusEnum.BEGINNING.getStatus()), "请先进行开始作业操作");

        Double feedbackSum = proFeedbackService.selectQuantityQualifySum(new ProFeedback(null, null, null, taskWorkunitId));
        double taskQuantityDouble = taskWorkunit.getQuantity().doubleValue();
        double canBeWaitQuantity = taskQuantityDouble - feedbackSum;
        ExceptionUtil.checkTrueThrowException(canBeWaitQuantity <= 0, "报工数量为0,不可报工");

        BigDecimal quantityQualify = feedback.getQuantityQualify();
        BigDecimal quantityUnQualify = feedback.getQuantityUnqualify();
        BigDecimal quantityFeedback = quantityQualify.add(quantityUnQualify);
        Double quantityFeedbackDouble = quantityFeedback.doubleValue();
        ExceptionUtil.checkTrueThrowException(quantityFeedbackDouble > canBeWaitQuantity, "报工数量大于可报工数量");

        ProTask task = proTaskService.selectBaseById(proTaskAssistProcess.getTaskId());
        BigDecimal taskQuantityWait = task.getQuantityWait();
        ExceptionUtil.checkTrueThrowException(quantityFeedbackDouble > taskQuantityWait.longValue()
                || quantityFeedbackDouble > taskQuantityDouble, "可加工数量不足");

        BigDecimal quantityWait = taskQuantityWait.subtract(quantityQualify);
        task.setQuantityWait(quantityWait);
        task.setQuantityQualify(task.getQuantityQualify().add(quantityQualify));
        task.setQuantityUnqualify(task.getQuantityUnqualify().add(quantityUnQualify));
        task.setQuantityProduced(task.getQuantityProduced().add(quantityQualify));

        taskWorkunit.setQuantityQualify(taskWorkunit.getQuantityQualify().add(quantityQualify));
        taskWorkunit.setQuantityUnqualify(taskWorkunit.getQuantityUnqualify().add(quantityUnQualify));
        taskWorkunit.setQuantityProduced(taskWorkunit.getQuantityProduced().add(quantityQualify));

        if (feedback.getCloseType() == 1 || quantityWait.doubleValue() == 0) {
            taskWorkunit.setStatus(TaskWorkunitStatusEnum.FINISHED.getStatus());
            proTaskWorkunitService.updateProTaskWorkunit(taskWorkunit);
        }

        //暂停更新记录
        if (Objects.equals(taskWorkunit.getStatus(), FINISHED.getStatus())) {
            proStartWorkService.closeObj(feedback);
        } else {
            proStartWorkService.updateObj(feedback);
        }

        //按派工表保存报工记录
        feedback.setWorkorderId(proTaskAssistProcess.getWorkorderId());
        proFeedbackService.assistFeedBack(feedback, proTaskAssistProcess);


        //查询关联下道工序
        ProTask nextTask = proTaskService.getNextTaskByTaskId(feedback.getTaskId());
        if (nextTask != null) {
            nextTask.setQuantityWait(DataUtil.getNormalData(nextTask.getQuantityWait()));

            // 下道任务待加工数累加合格数量
            if (quantityQualify.doubleValue() > 0) {
                nextTask.setQuantityWait(nextTask.getQuantityWait().add(quantityQualify));
            }
//
//            if (quantityUnQualify.doubleValue() > 0) {
//                nextTask.setQuantityWait(nextTask.getQuantityWait().add(quantityUnQualify));
//            }

            List<ProTaskWorkunit> proTaskWorkunits = proTaskWorkunitMapper.selectListByQw(new QueryWrapper<ProTaskWorkunit>().eq("task_id", nextTask.getTaskId()));
            proTaskWorkunits.stream().filter(x -> Objects.equals(x.getStatus(), FINISHED.getStatus())).forEach(x -> {
                x.setStatus(BEGINNING.getStatus());
                proTaskWorkunitService.updateProTaskWorkunit(x);
            });
            proTaskService.updateProTask(nextTask);
        }
        proTaskService.updateProTask(task);


        ProWorkorder proWorkorder = proWorkorderMapper.selectWorkorderById(proTaskAssistProcess.getWorkorderId());
        proWorkorder.setQuantityQualify(proWorkorder.getQuantityQualify().add(quantityQualify));
        proWorkorder.setQuantityUnqualify(proWorkorder.getQuantityUnqualify().add(quantityUnQualify));
        proWorkorder.setQuantityProduced(DataUtil.getNormalData(proWorkorder.getQuantityProduced()).add(quantityQualify));
        proWorkorderService.updateProWorkorder(proWorkorder);

        SapServiceAssistProcess sapServiceAssistProcess = new SapServiceAssistProcess(feedback, proTaskAssistProcess);
        sapServiceAssistProcess.execute();

        return 1;
    }


    // 检查是否满足外协数量
    public void checkAssistQuantity(ProTask task, ProTaskAssistProcess assistProcess) {
        Double sum = proFeedbackService.selectQuantityQualifySum(new ProFeedback(null, null, task.getTaskId(), null));
        ExceptionUtil.checkTrueThrowException((task.getQuantity().doubleValue() - sum) < assistProcess.getQuantity().doubleValue() || task.getQuantityWait().doubleValue() < assistProcess.getQuantity().doubleValue(), "可报工数不足");
    }


    @Transactional
    @Override
    public int addObj(ProTaskAssistProcess addAssistProcess) {
        ProTaskAssistProcess qo = new ProTaskAssistProcess();
        qo.setAssistProcessCode(addAssistProcess.getAssistProcessCode());
        qo.setAssistProcessItemCode(addAssistProcess.getAssistProcessItemCode());
        ExceptionUtil.checkTrueThrowException(StringUtils.isNotEmpty(this.selectBaseByObj(qo)), "外协编码已经生成外协任务");

        String workorderCode = addAssistProcess.getWorkorderCode();
        String processCode = addAssistProcess.getProcessCode();
        List<ProWorkorder> proWorkorders = this.proWorkorderMapper.selectListByQw(new QueryWrapper<ProWorkorder>().eq("workorder_code", workorderCode));
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(proWorkorders), "工单不存在");


        List<ProProcess> proProcessList = this.proProcessMapper.selectListByQw(new QueryWrapper<ProProcess>().eq("process_code", processCode));
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(proProcessList), "工序不存在");


        ProWorkorder proWorkorder = proWorkorders.get(0);
        ProProcess proProcess = proProcessList.get(0);
        Long processId = proProcess.getProcessId();


        Long workorderId = proWorkorder.getWorkorderId();
        List<ProWorkOrderProcessVo> proWorkOrderProcessVos = proWorkOrderProcessService.selectProWorkOrderProcessList(new ProWorkOrderProcess(workorderId, processId));
        ExceptionUtil.checkTrueThrowException(CollectionUtil.isEmpty(proWorkOrderProcessVos), "工序任务不存在");
        ProWorkOrderProcessVo processVo = proWorkOrderProcessVos.get(0);


        List<ProTask> proTasks = proTaskMapper.selectListByQw(new QueryWrapper<ProTask>().eq("t1.arrange_code", proWorkorder.getArrangeCode()).eq("t1.process_code", processCode));
        Optional<ProTask> first = proTasks.stream().filter(x -> Objects.equals(x.getProcessId(), processId)).findFirst();
        ExceptionUtil.checkTrueThrowException(!first.isPresent(), "工序任务不存在");
        ProTask task = first.get();
        checkAssistQuantity(task, addAssistProcess);
        addAssistProcess.setTaskId(task.getTaskId());
        addAssistProcess.setTaskCode(task.getTaskCode());
        addAssistProcess.setWorkorderProcessId(processVo.getWorkorderProcessId());
        addAssistProcess.setIdx(processVo.getProcessRef());
        addAssistProcess.setProcessId(processId);
        addAssistProcess.setWorkorderId(workorderId);
        addAssistProcess.setWorkorderCode(workorderCode);

        ProTaskWorkunit taskWorkunit = new ProTaskWorkunit();
        taskWorkunit.setQuantity(BigDecimal.valueOf(addAssistProcess.getQuantity()));
        taskWorkunit.setWorkunitId(AssistProcessConst.WORKUNIT_ID);
        taskWorkunit.setWorkstationId(AssistProcessConst.WORKSTATION_ID);
        taskWorkunit.setWorkstationCode(AssistProcessConst.WORKSTATION_CODE);

        ProTaskVo proTaskVo = task.deepCopyObj(ProTaskVo.class);
        proTaskVo.setSplitTaskWorkunit(taskWorkunit);
        taskWorkunit = proTaskService.splitTask(proTaskVo, TaskConst.ASSIST_TASK);

        addAssistProcess.setTaskWorkunitId(taskWorkunit.getTaskWorkunitId());
        addAssistProcess.setWorkunitId(taskWorkunit.getWorkunitId());
        addAssistProcess.setApiFlag(true);
        this.insertProTaskAssistProcess(addAssistProcess);
        return 1;
    }

}
