package com.bsn.mes.qc.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.mes.qc.domain.QcIqcLine;
import com.bsn.mes.qc.mapper.QcIqcLineMapper;
import com.bsn.mes.qc.service.IQcIqcLineService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

import static com.bsn.mes.constant.QcIqcConst.*;

/**
 * 来料检验单行Service业务层处理
 *
 * @date 2022-05-19
 */
@Service
public class QcIqcLineServiceImpl implements IQcIqcLineService {
    @Resource
    private QcIqcLineMapper qcIqcLineMapper;

    /**
     * 查询来料检验单行
     *
     * @param lineId 来料检验单行主键
     * @return 来料检验单行
     */
    @Override
    public QcIqcLine selectQcIqcLineByLineId(Long lineId) {
        return qcIqcLineMapper.selectQcIqcLineByLineId(lineId);
    }

    /**
     * 查询来料检验单行列表
     *
     * @param qcIqcLine 来料检验单行
     * @return 来料检验单行
     */
    @Override
    public List<QcIqcLine> selectQcIqcLineList(QcIqcLine qcIqcLine) {

        QueryWrapper<QcIqcLine> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(qcIqcLine.getIqcId()), "t2.iqc_code", qcIqcLine.getIqcCode());
        queryWrapper.eq(StringUtils.isNotEmpty(qcIqcLine.getSapItemCode()), "mi.sap_item_code", qcIqcLine.getSapItemCode());
        return qcIqcLineMapper.selectListByQw(queryWrapper);
    }

    /**
     * 新增来料检验单行
     *
     * @param qcIqcLine 来料检验单行
     * @return 结果
     */
    @Override
    public int insertQcIqcLine(QcIqcLine qcIqcLine) {
        qcIqcLine.createAction();
        return qcIqcLineMapper.insertQcIqcLine(qcIqcLine);
    }

    /**
     * 修改来料检验单行
     *
     * @param qcIqcLine 来料检验单行
     * @return 结果
     */
    @Override
    public int updateQcIqcLine(QcIqcLine qcIqcLine) {
        qcIqcLine.updateAction();
        return qcIqcLineMapper.updateQcIqcLine(qcIqcLine);
    }

    /**
     * 修改来料检验单行
     *
     * @param qcIqcLine 来料检验单行
     * @return 结果
     */
    @Override
    public int updateObj(QcIqcLine qcIqcLine) {
        Integer status = qcIqcLine.getStatus();
        String statusStr = String.valueOf(status);
        switch (statusStr) {
            case SUCCESS:
                break;
            case FAILD:
            case CANCEL:
                break;
            default:
                break;
        }
        return this.updateQcIqcLine(qcIqcLine);
    }

    @Override
    public int updateCrMajMinQuantity(Long iqcId, Long lineId) {
        QcIqcLine qcIqcLine = new QcIqcLine();
        qcIqcLine.setIqcId(iqcId);
        qcIqcLine.setLineId(lineId);
        return qcIqcLineMapper.updateCrMajMinQuantity(qcIqcLine);
    }

    /**
     * 批量删除来料检验单行
     *
     * @param lineIds 需要删除的来料检验单行主键
     * @return 结果
     */
    @Override
    public int deleteQcIqcLineByLineIds(Long[] lineIds) {
        return qcIqcLineMapper.deleteQcIqcLineByLineIds(lineIds);
    }

    @Override
    public int deleteByIqcId(Long iqcId) {
        return qcIqcLineMapper.deleteByIqcId(iqcId);
    }

    /**
     * 删除来料检验单行信息
     *
     * @param lineId 来料检验单行主键
     * @return 结果
     */
    @Override
    public int deleteQcIqcLineByLineId(Long lineId) {
        return qcIqcLineMapper.deleteQcIqcLineByLineId(lineId);
    }
}
