package com.bsn.mes.sap.execute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.utils.data.DataUtil;
import com.bsn.common.utils.data.ExceptionUtil;
import com.bsn.common.utils.spring.SpringUtils;
import com.bsn.mes.constant.WorkorderConst;
import com.bsn.mes.pro.domain.ProFeedback;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.sap.SAPDtoFeedback;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkorderMapper;
import com.bsn.mes.sap.SapService;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Objects;

@Data
@Slf4j
@Component
@AllArgsConstructor
@NoArgsConstructor
public class SapServiceFeedback {
    private SAPDtoFeedback sapDtoFeedback;
    private ProFeedback feedback;
    private String actionMsg;

    public void execute() {
        ProWorkorderMapper proWorkorderMapper = SpringUtils.getBean(ProWorkorderMapper.class);
        SapService sapService = SpringUtils.getBean(SapService.class);
        ProWorkorder proWorkorder = proWorkorderMapper.selectListByQw(new QueryWrapper<ProWorkorder>().eq("workorder_id", feedback.getWorkorderId())).get(0);
        if (Objects.equals(proWorkorder.getWorkorderType(), WorkorderConst.WORK_ORDER_COMPLEMENTS_TYPE)) {
            if (sapDtoFeedback.getWorkorderCode().contains("-")) {
                sapDtoFeedback.setWorkorderCode(sapDtoFeedback.getWorkorderCode().split("-")[0]);
            }
        }

        String json = JSON.toJSONString(Collections.singletonList(sapDtoFeedback));
        log.info("------------------------------------------------------------------->>>> sap操作:  " + actionMsg);
        log.info("------------------------------------------------------------------->>>> 请求参数:  " + json);
        String result = sapService.uploadFeedbackData(json);
        log.info("------------------------------------------------------------------->>>> 返回结果:  " + result);
        feedback.setSapRequestParam(json);
        feedback.setSapResponse(result);
        List<JSONObject> jsonObjects = null;
        // String abc = "[{\"TYPE\":\"E\",\"ID\":\"\",\"NUMBER\":\"000\",\"MESSAGE\":\"订单报工成功\",\"LOG_NO\":\"\",\"LOG_MSG_NO\":\"000000\",\"MESSAGE_V1\":\"0000000037\",\"MESSAGE_V2\":\"00000002\",\"MESSAGE_V3\":\"\",\"MESSAGE_V4\":\"\",\"PARAMETER\":\"\",\"ROW\":0 ,\"FIELD\":\"\",\"SYSTEM\":\"\"}]";
        //
        try {
            jsonObjects = JSON.parseArray(result, JSONObject.class);
            JSONObject jsonObject = jsonObjects.get(0);
            Object type = jsonObject.get("TYPE");
            Object msg = jsonObject.get("MESSAGE");

            ExceptionUtil.checkTrueThrowException(type == null, "sap报工失败!");
            ExceptionUtil.checkTrueThrowException(!"s".equalsIgnoreCase(type.toString()), msg.toString());


            String sapFeedbackCode = DataUtil.getNormalData(jsonObject.get("MESSAGE_V1"), String.class);
            String sapFeedbackItemCode = DataUtil.getNormalData(jsonObject.get("MESSAGE_V2"), String.class);
            feedback.setSapFeedbackCode(sapFeedbackCode);
            feedback.setSapFeedbackItemCode(sapFeedbackItemCode);
        } catch (Exception e) {
            ExceptionUtil.throwExcetion(e.getMessage());
        }

    }
}
