package com.bsn.mes.util;

import com.bsn.common.utils.data.DataUtil;
import com.bsn.common.utils.reflect.PackageUtil;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.bind.annotation.RequestMapping;

import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.annotation.*;
import java.lang.reflect.Method;
import java.util.*;

public class ApiPrintUtil {

    // 定义一个简单的注解
    @Retention(RetentionPolicy.RUNTIME)
    @Target(ElementType.METHOD)
    public @interface CustomAnnotation {
        String value();
    }

    // 使用注解的方法
    @CustomAnnotation(value = "Hello")
    public void annotatedMethod() {
        // 方法实现
    }

//    public static void main(String[] args) {
//        try {
//            // 获取main方法所在的类的Class对象
//            Class<?> clazz = Class.forName("AnnotationExample");
//            // 获取annotatedMethod方法的Method对象
//            Method method = clazz.getMethod("annotatedMethod");
//            // 获取方法上的注解
//            Annotation[] annotations = method.getAnnotations();
//            for (Annotation annotation : annotations) {
//                if (annotation instanceof CustomAnnotation) {
//                    CustomAnnotation customAnnotation = (CustomAnnotation) annotation;
//                    System.out.println("Found annotation: " + customAnnotation.value());
//                }
//            }
//        } catch (ClassNotFoundException | NoSuchMethodException e) {
//            e.printStackTrace();
//        }
//    }
//

    private void getMethod(String pkg) throws ClassNotFoundException {
        Class<?> clazz = Class.forName(pkg);
        Method[] methods = clazz.getMethods();


        for (Method method : methods) {
            System.out.println(method.getName());
        }
    }

    public static void main(String[] args) {
        getAllClass();
    }

    public static void getAllClass() {

        List<Map<String, Object>> objects = new ArrayList<>();
        Package pack = Package.getPackage("com.bsn.mes.restful"); // 替换为你的包名
        Set<Class<?>> allClasses = PackageUtil.getAllClasses(pack);
        for (Class<?> allClass : allClasses) {
            // 获取main方法所在的类的Class对象
//            Class<?> clazz = Class.forName("AnnotationExample");
            // 获取annotatedMethod方法的Method对象
//                    Method method = allClass.getMethod("annotatedMethod");
            Method[] methods = allClass.getMethods();
            for (Method method : methods) {
                Map<String, Object> map = new HashMap<>();
                // 获取方法上的注解
                Annotation[] annotations = method.getAnnotations();
                for (Annotation annotation : annotations) {
                    if (annotation instanceof ApiOperation) {
                        ApiOperation anno1 = (ApiOperation) annotation;
                        map.put("apidesc", anno1.value());
                    }
                    if (annotation instanceof RequestMapping) {
                        RequestMapping anno2 = (RequestMapping) annotation;
                        map.put("apiaddr", anno2.value());
                        map.put("apimethod", anno2.method().toString());
                    }
                }
                objects.add(map);
            }
            excel(objects, "C:\\Users\\1\\Desktop\\对外提供接口地址2.xlsx");
        }
    }


    public static void excel(List<Map<String, Object>> list, String filepath) {
        Set<String> objects = new HashSet<>();

        List<Map<String, Object>> objects1 = new ArrayList<>();
        for (Map<String, Object> entry : list) {
            Object o = entry.get("apiaddr");
            if (o != null && o != "") {
                if (objects.add(o.toString())) {
                    objects1.add(entry);
                }
            }

        }

        // 创建一个新的工作簿
        Workbook workbook = new XSSFWorkbook();
        // 创建一个工作表
        Sheet sheet = workbook.createSheet("接口");

        // 创建行（0基索引）
        Row row = sheet.createRow(0);

        // 创建单元格并设置值
        Cell cell = row.createCell(0);
        cell.setCellValue("服务启用");
        cell = row.createCell(3);
        cell.setCellValue("接口服务描述");
        cell = row.createCell(1);
        cell.setCellValue("接口地址");
        cell = row.createCell(2);
        cell.setCellValue("api地址");

        int rowIndex = 1;
        for (Map<String, Object> dataRow : objects1) {
            // 创建行（0基索引）
            row = sheet.createRow(rowIndex);


            // 创建单元格并设置值
            cell = row.createCell(0);
            cell.setCellValue("是");
            cell = row.createCell(3);
            cell.setCellValue(DataUtil.getNormalData(dataRow.get("apidesc"), String.class));
            cell = row.createCell(1);
            cell.setCellValue("http://192.168.3.91:8080");
            cell = row.createCell(2);
            cell.setCellValue(DataUtil.getNormalData(dataRow.get("apiaddr"), String.class));
            rowIndex++;
        }

        // 写入到文件
        try (FileOutputStream outputStream = new FileOutputStream(filepath)) {
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // 清理资源
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }
}