package com.bs.mes.schedule.strategy;

import com.bsn.mes.pro.schedule.Equipment;
import com.bsn.mes.pro.schedule.IOperationTimeCalculator;
import com.bsn.mes.pro.schedule.Job;
import com.bsn.mes.pro.schedule.TaskSchedulingContext;
import com.bsn.mes.pro.schedule.strategy.EvaluateEquipmentDeliveryTime;
import com.bsn.mes.pro.schedule.strategy.EvaluateEquipmentShift;
import org.junit.Assert;
import org.junit.Test;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;

public class EvaluateEquipmentDeliveryTimeTest {
    @Test public void evaluate(){
        EvaluateEquipmentDeliveryTime evaluateEquipmentDeliveryTime = new EvaluateEquipmentDeliveryTime();
        TaskSchedulingContext context = new TaskSchedulingContext();
        IOperationTimeCalculator.OperationTimePlan operationTimePlan = new IOperationTimeCalculator.OperationTimePlan();
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItemList = new ArrayList<IOperationTimeCalculator.OperationTimePlanItem>();
        operationTimePlan.setPlan(operationTimePlanItemList);
        operationTimePlan.setScheduledSuccess(true);
        operationTimePlanItemList.add(new IOperationTimeCalculator.OperationTimePlanItem(){{
            setStartedTime(Duration.ofMinutes(2888));
            setEndedTime(Duration.ofMinutes(2898));
            setShiftType("白班");
        }});
        Job job = new Job("1", Duration.ofMinutes(12));
        job.setProducedDeliveryTime(Duration.ofMinutes(3000));
        context.setJob(job);
        List<TaskSchedulingContext.TaskSchedulingResultContext> scheduleResult = new ArrayList<>();
        Equipment equipment1 = new Equipment();
        equipment1.setId(1l);
        context.setEquipment(equipment1);
        scheduleResult.add(new TaskSchedulingContext.TaskSchedulingResultContext(equipment1, operationTimePlan, context));
        //设备在交期范围内的加工
        double rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan, scheduleResult);
        Assert.assertEquals(10, rst, Double.valueOf(0.0));
        //设备在交期范围外的加工
        operationTimePlanItemList.add(new IOperationTimeCalculator.OperationTimePlanItem(){{
            setStartedTime(Duration.ofMinutes(3000));
            setEndedTime(Duration.ofMinutes(3002));
            setShiftType("白班");
        }});
        rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan, scheduleResult);
        Assert.assertEquals(5, rst, Double.valueOf(0.001));
        //我设备可选情况
        Equipment equipment2 = new Equipment();
        equipment2.setId(2l);
        IOperationTimeCalculator.OperationTimePlan operationTimePlan2 = new IOperationTimeCalculator.OperationTimePlan();
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItemList2 = new ArrayList<IOperationTimeCalculator.OperationTimePlanItem>();
        operationTimePlan2.setPlan(operationTimePlanItemList2);
        operationTimePlan2.setScheduledSuccess(true);
        operationTimePlanItemList2.add(new IOperationTimeCalculator.OperationTimePlanItem(){{
            setStartedTime(Duration.ofMinutes(2888));
            setEndedTime(Duration.ofMinutes(3898));
            setShiftType("白班");
        }});
        scheduleResult.add(new TaskSchedulingContext.TaskSchedulingResultContext(equipment2, operationTimePlan2, context));
        Equipment equipment3 = new Equipment();

        IOperationTimeCalculator.OperationTimePlan operationTimePlan3 = new IOperationTimeCalculator.OperationTimePlan();
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItemList3 = new ArrayList<IOperationTimeCalculator.OperationTimePlanItem>();
        operationTimePlan3.setPlan(operationTimePlanItemList3);
        operationTimePlan3.setScheduledSuccess(true);
        operationTimePlanItemList3.add(new IOperationTimeCalculator.OperationTimePlanItem(){{
            setStartedTime(Duration.ofMinutes(2588));
            setEndedTime(Duration.ofMinutes(2598));
            setShiftType("白班");
        }});
        equipment3.setId(3l);
        scheduleResult.add(new TaskSchedulingContext.TaskSchedulingResultContext(equipment3, operationTimePlan3, context));

        operationTimePlanItemList.get(1).setShiftType("中班");
        rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan, scheduleResult);
        Assert.assertEquals(4, rst, Double.valueOf(0.001));
    }
}
